# Generated by Django 2.2.13 on 2020-09-23 13:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0024_related_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='optionset',
            name='provider_key',
            field=models.SlugField(blank=True, help_text='The provider for this optionset. If set, it will create dynamic options for this optionset.', max_length=128, verbose_name='Provider'),
        ),
    ]



import pytest
from django.contrib.auth import get_user_model
from django.urls import reverse

users = (
    ('site', 'site'),
    ('api', 'api'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'site': 200, 'api': 200, 'user': 200, 'anonymous': 401
    },
    'detail': {
        'site': 200, 'api': 200, 'user': 404, 'anonymous': 401
    },
    'create': {
        'site': 405, 'api': 405, 'user': 405, 'anonymous': 401
    },
    'update': {
        'site': 405, 'api': 405, 'user': 405, 'anonymous': 401
    },
    'delete': {
        'site': 405, 'api': 405, 'user': 405, 'anonymous': 401
    }
}

urlnames = {
    'list': 'v1-accounts:user-list',
    'detail': 'v1-accounts:user-detail'
}


@pytest.mark.parametrize('username,password', users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()
    if response.status_code == 200:
        if username == 'api':
            assert len(response.json()) == 11
        elif username == 'site':
            # the site admin must not see the user 'other'
            assert len(response.json()) == 10
        else:
            assert len(response.json()) == 0


@pytest.mark.parametrize('username,password', users)
def test_detail(db, client, username, password):
    client.login(username=username, password=password)
    instances = get_user_model().objects.all()

    for instance in instances:
        url = reverse(urlnames['detail'], args=[instance.pk])
        response = client.get(url)
        if username == 'site' and instance.username == 'other':
            # the site admin must not see the user 'other'
            assert response.status_code == 404, response.json()
        else:
            assert response.status_code == status_map['detail'][username], response.json()


@pytest.mark.parametrize('username,password', users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {})
    assert response.status_code == status_map['create'][username], response.json()


@pytest.mark.parametrize('username,password', users)
def test_update(db, client, username, password):
    client.login(username=username, password=password)
    instances = get_user_model().objects.all()

    for instance in instances:
        url = reverse(urlnames['detail'], args=[instance.pk])
        response = client.put(url, {}, content_type='application/json')
        assert response.status_code == status_map['update'][username], response.json()


@pytest.mark.parametrize('username,password', users)
def test_delete(db, client, username, password):
    client.login(username=username, password=password)
    instances = get_user_model().objects.all()

    for instance in instances:
        url = reverse(urlnames['detail'], args=[instance.pk])
        response = client.delete(url)
        assert response.status_code == status_map['delete'][username], response.json()



from django.template import Context, Template, TemplateSyntaxError
from rest_framework import exceptions, serializers
from rest_framework.reverse import reverse

from rdmo.core.serializers import (MarkdownSerializerMixin, SiteSerializer,
                                   TranslationSerializerMixin)
from rdmo.core.utils import get_language_warning

from ..models import View
from ..validators import ViewLockedValidator, ViewUniqueURIValidator


class ViewSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    key = serializers.SlugField(required=True)

    def validate(self, data):
        # try to render the tamplate to see that the syntax is ok
        try:
            Template(data['template']).render(Context({}))
        except (KeyError, IndexError):
            pass
        except (TemplateSyntaxError, TypeError) as e:
            raise exceptions.ValidationError({'template': '\n'.join(e.args)})

        return super(ViewSerializer, self).validate(data)

    class Meta:
        model = View
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'available',
            'catalogs',
            'sites',
            'groups',
            'template'
        )
        trans_fields = (
            'title',
            'help'
        )
        validators = (
            ViewUniqueURIValidator(),
            ViewLockedValidator()
        )


class ViewIndexSerializer(MarkdownSerializerMixin, serializers.ModelSerializer):

    markdown_fields = ('help', )

    sites = SiteSerializer(many=True, read_only=True)
    warning = serializers.SerializerMethodField()
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = View
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'locked',
            'available',
            'sites',
            'title',
            'help',
            'warning',
            'xml_url'
        )

    def get_warning(self, obj):
        return get_language_warning(obj, 'title')

    def get_xml_url(self, obj):
        return reverse('v1-views:view-detail-export', args=[obj.pk])



from django.conf import settings
from django.db import models
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from rdmo.core.exports import XMLResponse
from rdmo.core.permissions import HasModelPermission
from rdmo.core.views import ChoicesViewSet
from rdmo.core.viewsets import CopyModelMixin

from .models import Option, OptionSet
from .renderers import OptionRenderer, OptionSetRenderer
from .serializers.export import (OptionExportSerializer,
                                 OptionSetExportSerializer)
from .serializers.v1 import (OptionIndexSerializer, OptionSerializer,
                             OptionSetIndexSerializer,
                             OptionSetNestedSerializer, OptionSetSerializer)


class OptionSetViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = OptionSet.objects.order_by('order').prefetch_related(
        'conditions',
        'questions',
        'options'
    )
    serializer_class = OptionSetSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'uri',
        'key',
        'comment'
    )

    @action(detail=False)
    def nested(self, request):
        serializer = OptionSetNestedSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False)
    def index(self, request):
        serializer = OptionSetIndexSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = OptionSetExportSerializer(self.get_queryset(), many=True)
        xml = OptionSetRenderer().render(serializer.data)
        return XMLResponse(xml, name='optionsets')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = OptionSetExportSerializer(self.get_object())
        xml = OptionSetRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().key)


class OptionViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = Option.objects.order_by('optionset__order', 'order') \
                             .annotate(values_count=models.Count('values')) \
                             .annotate(projects_count=models.Count('values__project', distinct=True)) \
                             .select_related('optionset') \
                             .prefetch_related('conditions')
    serializer_class = OptionSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'uri',
        'key',
        'optionset',
        'comment'
    )

    @action(detail=False)
    def index(self, request):
        serializer = OptionIndexSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = OptionExportSerializer(self.get_queryset(), many=True)
        xml = OptionRenderer().render(serializer.data)
        return XMLResponse(xml, name='options')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = OptionExportSerializer(self.get_object())
        xml = OptionRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().path)


class ProviderViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = settings.OPTIONSET_PROVIDERS



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def run_data_migration(apps, schema_editor):
    Question = apps.get_model('questions', 'Question')

    for question in Question.objects.all():
        if question.value_type == 'options':
            question.value_type = 'option'

        question.save()


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0023_option'),
        ('projects', '0019_option'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Exists, OuterRef
from django.db.models.signals import pre_delete
from django.dispatch import receiver
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from mptt.models import MPTTModel, TreeForeignKey

from rdmo.core.models import Model
from rdmo.domain.models import Attribute
from rdmo.questions.models import Catalog, Question
from rdmo.tasks.models import Task
from rdmo.views.models import View

from ..managers import ProjectManager


class Project(MPTTModel, Model):

    objects = ProjectManager()

    parent = TreeForeignKey(
        'self', null=True, blank=True,
        on_delete=models.DO_NOTHING, related_name='children', db_index=True,
        verbose_name=_('Parent project'),
        help_text=_('The parent project of this project.')
    )
    user = models.ManyToManyField(
        settings.AUTH_USER_MODEL, through='Membership', related_name='projects',
        verbose_name=_('User'),
        help_text=_('The list of users for this project.')
    )
    site = models.ForeignKey(
        Site, on_delete=models.SET_NULL, null=True,
        verbose_name=_('Site'),
        help_text=_('The site this project belongs to (in a multi site setup).')
    )
    title = models.CharField(
        max_length=256,
        verbose_name=_('Title'),
        help_text=_('The title for this project.')
    )
    description = models.TextField(
        blank=True,
        verbose_name=_('Description'),
        help_text=_('A description for this project (optional).')
    )
    catalog = models.ForeignKey(
        Catalog, related_name='projects', on_delete=models.SET_NULL, null=True,
        verbose_name=_('Catalog'),
        help_text=_('The catalog which will be used for this project.')
    )
    tasks = models.ManyToManyField(
        Task, blank=True, through='Issue',
        verbose_name=_('Tasks'),
        help_text=_('The tasks that will be used for this project.')
    )
    views = models.ManyToManyField(
        View, blank=True,
        verbose_name=_('Views'),
        help_text=_('The views that will be used for this project.')
    )

    class Meta:
        ordering = ('tree_id', 'level', 'title')
        verbose_name = _('Project')
        verbose_name_plural = _('Projects')

    class MPTTMeta:
        order_insertion_by = ('title', )

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('project', kwargs={'pk': self.pk})

    def clean(self):
        if self.id and self.parent in self.get_descendants(include_self=True):
            raise ValidationError({
                'parent': [_('A project may not be moved to be a child of itself or one of its descendants.')]
            })

    @property
    def progress(self):
        # create a queryset for the attributes of the catalog for this project
        # the subquery is used to query only attributes which have a question in the catalog, which is not optional
        questions = Question.objects.filter(attribute_id=OuterRef('pk'), questionset__section__catalog_id=self.catalog.id) \
                                    .exclude(is_optional=True)
        attributes = Attribute.objects.annotate(active=Exists(questions)).filter(active=True).distinct()

        # query the total number of attributes from the qs above
        total = attributes.count()

        # query all current values with attributes from the qs above, but where the text, option, or file field is set,
        # and count only one value per attribute
        values = self.values.filter(snapshot=None) \
                            .filter(attribute__in=attributes) \
                            .exclude((models.Q(text='') | models.Q(text=None)) & models.Q(option=None) &
                                     (models.Q(file='') | models.Q(file=None))) \
                            .distinct().values('attribute').count()

        try:
            ratio = values / total
        except ZeroDivisionError:
            ratio = 0

        return {
            'total': total,
            'values': values,
            'ratio': ratio
        }

    @cached_property
    def member(self):
        return self.user.all()

    @cached_property
    def owners_str(self):
        return ', '.join(['' if x is None else str(x) for x in self.user.filter(membership__role='owner')])

    @cached_property
    def owners(self):
        return self.user.filter(memberships__role='owner')

    @cached_property
    def managers(self):
        return self.user.filter(memberships__role='manager')

    @cached_property
    def authors(self):
        return self.user.filter(memberships__role='author')

    @cached_property
    def guests(self):
        return self.user.filter(memberships__role='guest')

    @property
    def file_size(self):
        queryset = self.values.filter(snapshot=None).exclude(models.Q(file='') | models.Q(file=None))
        return sum([value.file.size for value in queryset])


@receiver(pre_delete, sender=Project)
def reparent_children(sender, instance, **kwargs):
    for child in instance.get_children():
        child.move_to(instance.parent, 'last-child')
        child.save()



from django.conf import settings
from rest_framework import serializers
from rest_framework.reverse import reverse

from rdmo.conditions.models import Condition
from rdmo.core.serializers import SiteSerializer, TranslationSerializerMixin
from rdmo.core.utils import get_language_warning
from rdmo.domain.models import Attribute
from rdmo.options.models import OptionSet

from ..models import Catalog, Question, QuestionSet, Section
from ..validators import (CatalogLockedValidator, CatalogUniqueURIValidator,
                          QuestionLockedValidator, QuestionSetLockedValidator,
                          QuestionSetQuestionSetValidator,
                          QuestionSetUniqueURIValidator,
                          QuestionUniqueURIValidator, SectionLockedValidator,
                          SectionUniqueURIValidator)
from ..utils import get_widget_type_choices


class CatalogSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    key = serializers.SlugField(required=True)
    projects_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = Catalog
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'order',
            'available',
            'sites',
            'groups',
            'projects_count'
        )
        trans_fields = (
            'title',
            'help'
        )
        validators = (
            CatalogUniqueURIValidator(),
            CatalogLockedValidator()
        )


class SectionSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    key = serializers.SlugField(required=True)

    class Meta:
        model = Section
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'catalog',
            'order',
        )
        trans_fields = (
            'title',
        )
        validators = (
            SectionUniqueURIValidator(),
            SectionLockedValidator()
        )


class QuestionSetSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    key = serializers.SlugField(required=True)

    class Meta:
        model = QuestionSet
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'attribute',
            'section',
            'questionset',
            'is_collection',
            'order',
            'conditions',
        )
        trans_fields = (
            'title',
            'help',
            'verbose_name',
            'verbose_name_plural',
        )
        validators = (
            QuestionSetUniqueURIValidator(),
            QuestionSetQuestionSetValidator(),
            QuestionSetLockedValidator()
        )


class QuestionSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    key = serializers.SlugField(required=True)
    widget_type = serializers.ChoiceField(choices=get_widget_type_choices(), required=True)

    class Meta:
        model = Question
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'attribute',
            'questionset',
            'is_collection',
            'is_optional',
            'order',
            'maximum',
            'minimum',
            'step',
            'default_option',
            'default_external_id',
            'widget_type',
            'value_type',
            'unit',
            'width',
            'optionsets',
            'conditions'
        )
        trans_fields = (
            'text',
            'help',
            'default_text',
            'verbose_name',
            'verbose_name_plural',
        )
        validators = (
            QuestionUniqueURIValidator(),
            QuestionLockedValidator()
        )

    def to_internal_value(self, data):
        # handles an empty width, maximum, minimum, or step field
        for field in ['width', 'maximum', 'minimum', 'step']:
            if data.get(field) == '':
                data[field] = None

        return super().to_internal_value(data)


class CatalogIndexSerializer(serializers.ModelSerializer):

    class Meta:
        model = Catalog
        fields = (
            'id',
            'title',
            'uri',
            'key'
        )


class SectionIndexSerializer(serializers.ModelSerializer):

    class Meta:
        model = Section
        fields = (
            'id',
            'title',
            'uri',
            'path'
        )


class QuestionSetIndexSerializer(serializers.ModelSerializer):

    class Meta:
        model = QuestionSet
        fields = (
            'id',
            'title',
            'uri',
            'path',
        )


class QuestionIndexSerializer(serializers.ModelSerializer):

    class Meta:
        model = Question
        fields = (
            'id',
            'text',
            'uri',
            'path'
        )


class AttributeNestedSerializer(serializers.ModelSerializer):

    class Meta:
        model = Attribute
        fields = (
            'id',
            'uri'
        )


class OptionSetNestedSerializer(serializers.ModelSerializer):

    class Meta:
        model = OptionSet
        fields = (
            'id',
            'uri'
        )


class ConditionNestedSerializer(serializers.ModelSerializer):

    class Meta:
        model = Condition
        fields = (
            'id',
            'uri'
        )


class QuestionNestedSerializer(serializers.ModelSerializer):

    warning = serializers.SerializerMethodField()
    attribute = AttributeNestedSerializer(read_only=True)
    conditions = ConditionNestedSerializer(many=True, read_only=True)
    optionsets = OptionSetNestedSerializer(read_only=True, many=True)
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = Question
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'path',
            'locked',
            'order',
            'text',
            'attribute',
            'conditions',
            'optionsets',
            'is_collection',
            'is_optional',
            'warning',
            'xml_url'
        )

    def get_warning(self, obj):
        return get_language_warning(obj, 'text')

    def get_xml_url(self, obj):
        return reverse('v1-questions:question-detail-export', args=[obj.pk])


class QuestionSetNestedSerializer(serializers.ModelSerializer):

    questionsets = serializers.SerializerMethodField()
    questions = QuestionNestedSerializer(many=True, read_only=True)
    warning = serializers.SerializerMethodField()
    attribute = AttributeNestedSerializer(read_only=True)
    conditions = ConditionNestedSerializer(many=True, read_only=True)
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = QuestionSet
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'path',
            'locked',
            'order',
            'title',
            'attribute',
            'conditions',
            'is_collection',
            'section',
            'questionset',
            'questionsets',
            'questions',
            'warning',
            'xml_url'
        )

    def get_questionsets(self, obj):
        queryset = obj.questionsets.all()
        serializer = QuestionSetNestedSerializer(queryset, many=True)
        return serializer.data

    def get_warning(self, obj):
        return get_language_warning(obj, 'title')

    def get_xml_url(self, obj):
        return reverse('v1-questions:questionset-detail-export', args=[obj.pk])


class SectionNestedSerializer(serializers.ModelSerializer):

    questionsets = QuestionSetNestedSerializer(many=True)
    warning = serializers.SerializerMethodField()
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = Section
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'path',
            'locked',
            'order',
            'title',
            'questionsets',
            'warning',
            'xml_url'
        )

    def get_warning(self, obj):
        return get_language_warning(obj, 'title')

    def get_xml_url(self, obj):
        return reverse('v1-questions:section-detail-export', args=[obj.pk])


class CatalogNestedSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    sections = SectionNestedSerializer(many=True, read_only=True)
    sites = SiteSerializer(many=True, read_only=True)
    warning = serializers.SerializerMethodField()
    xml_url = serializers.SerializerMethodField()
    export_urls = serializers.SerializerMethodField()
    projects_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = Catalog
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'locked',
            'order',
            'sites',
            'title',
            'help',
            'sections',
            'warning',
            'xml_url',
            'export_urls',
            'projects_count'
        )
        trans_fields = (
            'title',
        )

    def get_warning(self, obj):
        return get_language_warning(obj, 'title')

    def get_xml_url(self, obj):
        return reverse('v1-questions:catalog-detail-export', args=[obj.pk])

    def get_export_urls(self, obj):
        urls = {}
        for key, text in settings.EXPORT_FORMATS:
            urls[key] = reverse('questions_catalog_export', args=[obj.pk, key])
        return urls



from rdmo.core.renderers import BaseXMLRenderer
from rdmo.core.utils import get_languages


class XMLRenderer(BaseXMLRenderer):

    def render_catalog(self, xml, catalog):
        xml.startElement('catalog', {'dc:uri': catalog['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, catalog['uri_prefix'])
        self.render_text_element(xml, 'key', {}, catalog['key'])
        self.render_text_element(xml, 'dc:comment', {}, catalog['comment'])
        self.render_text_element(xml, 'order', {}, catalog['order'])

        for lang_code, lang_string, lang_field in get_languages():
            self.render_text_element(xml, 'title', {'lang': lang_code}, catalog['title_%s' % lang_code])
            self.render_text_element(xml, 'help', {'lang': lang_code}, catalog['help_%s' % lang_code])

        xml.endElement('catalog')

        if 'sections' in catalog and catalog['sections']:
            for section in catalog['sections']:
                self.render_section(xml, section)

    def render_section(self, xml, section):
        xml.startElement('section', {'dc:uri': section['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, section['uri_prefix'])
        self.render_text_element(xml, 'key', {}, section['key'])
        self.render_text_element(xml, 'path', {}, section['path'])
        self.render_text_element(xml, 'dc:comment', {}, section['comment'])
        self.render_text_element(xml, 'catalog', {'dc:uri': section['catalog']}, None)
        self.render_text_element(xml, 'order', {}, section['order'])

        for lang_code, lang_string, lang_field in get_languages():
            self.render_text_element(xml, 'title', {'lang': lang_code}, section['title_%s' % lang_code])

        xml.endElement('section')

        if 'questionsets' in section and section['questionsets']:
            for questionset in section['questionsets']:
                self.render_questionset(xml, questionset)

    def render_questionset(self, xml, questionset):
        xml.startElement('questionset', {'dc:uri': questionset['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, questionset['uri_prefix'])
        self.render_text_element(xml, 'key', {}, questionset['key'])
        self.render_text_element(xml, 'path', {}, questionset['path'])
        self.render_text_element(xml, 'dc:comment', {}, questionset['comment'])
        self.render_text_element(xml, 'attribute', {'dc:uri': questionset['attribute']}, None)
        self.render_text_element(xml, 'section', {'dc:uri': questionset['section']}, None)
        self.render_text_element(xml, 'questionset', {'dc:uri': questionset['questionset']}, None)
        self.render_text_element(xml, 'is_collection', {}, questionset['is_collection'])
        self.render_text_element(xml, 'order', {}, questionset['order'])

        for lang_code, lang_string, lang_field in get_languages():
            self.render_text_element(xml, 'title', {'lang': lang_code}, questionset['title_%s' % lang_code])
            self.render_text_element(xml, 'help', {'lang': lang_code}, questionset['help_%s' % lang_code])
            self.render_text_element(xml, 'verbose_name', {'lang': lang_code}, questionset['verbose_name_%s' % lang_code])
            self.render_text_element(xml, 'verbose_name_plural', {'lang': lang_code}, questionset['verbose_name_plural_%s' % lang_code])

        xml.startElement('conditions', {})
        if 'conditions' in questionset and questionset['conditions']:
            for condition in questionset['conditions']:
                self.render_text_element(xml, 'condition', {'dc:uri': condition}, None)
        xml.endElement('conditions')

        xml.endElement('questionset')

        if 'questionsets' in questionset and questionset['questionsets']:
            for qs in questionset['questionsets']:
                self.render_questionset(xml, qs)

        if 'questions' in questionset and questionset['questions']:
            for question in questionset['questions']:
                self.render_question(xml, question)

    def render_question(self, xml, question):
        xml.startElement('question', {'dc:uri': question['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, question['uri_prefix'])
        self.render_text_element(xml, 'key', {}, question['key'])
        self.render_text_element(xml, 'path', {}, question['path'])
        self.render_text_element(xml, 'dc:comment', {}, question['comment'])
        self.render_text_element(xml, 'attribute', {'dc:uri': question['attribute']}, None)
        self.render_text_element(xml, 'questionset', {'dc:uri': question['questionset']}, None)
        self.render_text_element(xml, 'is_collection', {}, question['is_collection'])
        self.render_text_element(xml, 'is_optional', {}, question['is_optional'])
        self.render_text_element(xml, 'order', {}, question['order'])

        for lang_code, lang_string, lang_field in get_languages():
            self.render_text_element(xml, 'help', {'lang': lang_code}, question['help_%s' % lang_code])
            self.render_text_element(xml, 'text', {'lang': lang_code}, question['text_%s' % lang_code])
            self.render_text_element(xml, 'default_text', {'lang': lang_code}, question['default_text_%s' % lang_code])
            self.render_text_element(xml, 'verbose_name', {'lang': lang_code}, question['verbose_name_%s' % lang_code])
            self.render_text_element(xml, 'verbose_name_plural', {'lang': lang_code}, question['verbose_name_plural_%s' % lang_code])

        self.render_text_element(xml, 'default_option', {'dc:uri': question['default_option']}, None)
        self.render_text_element(xml, 'default_external_id', {}, question['default_external_id'])

        self.render_text_element(xml, 'widget_type', {}, question['widget_type'])
        self.render_text_element(xml, 'value_type', {}, question['value_type'])
        self.render_text_element(xml, 'maximum', {}, question['maximum'])
        self.render_text_element(xml, 'minimum', {}, question['minimum'])
        self.render_text_element(xml, 'step', {}, question['step'])
        self.render_text_element(xml, 'unit', {}, question['unit'])
        self.render_text_element(xml, 'width', {}, question['width'])

        xml.startElement('optionsets', {})
        if 'optionsets' in question and question['optionsets']:
            for optionset in question['optionsets']:
                self.render_text_element(xml, 'optionset', {'dc:uri': optionset}, None)
        xml.endElement('optionsets')

        xml.startElement('conditions', {})
        if 'conditions' in question and question['conditions']:
            for condition in question['conditions']:
                self.render_text_element(xml, 'condition', {'dc:uri': condition}, None)
        xml.endElement('conditions')

        xml.endElement('question')


class CatalogRenderer(XMLRenderer):

    def render_document(self, xml, catalogs):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for catalog in catalogs:
            self.render_catalog(xml, catalog)
        xml.endElement('rdmo')


class SectionRenderer(XMLRenderer):

    def render_document(self, xml, sections):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for section in sections:
            self.render_section(xml, section)
        xml.endElement('rdmo')


class QuestionSetRenderer(XMLRenderer):

    def render_document(self, xml, questionsets):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for questionset in questionsets:
            self.render_questionset(xml, questionset)
        xml.endElement('rdmo')


class QuestionRenderer(XMLRenderer):

    def render_document(self, xml, questions):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for question in questions:
            self.render_question(xml, question)
        xml.endElement('rdmo')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-01 08:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0015_permissions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='catalog',
            field=models.ForeignKey(help_text='The catalog which will be used for this project.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='questions.Catalog', verbose_name='Catalog'),
        ),
    ]



import pytest

users = (
    ('admin', 'admin'),
    ('user', 'user'),
    ('anonymous', None),
)


@pytest.mark.parametrize("username,password", users)
def test_swagger(db, client, username, password):
    client.login(username=username, password=password)

    response = client.get('/api/v1/')

    if password:
        assert response.status_code == 200
    else:
        assert response.status_code == 302



# Generated by Django 2.2rc1 on 2019-03-26 13:29

from django.db import migrations
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0042_remove_null_true'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='attribute',
            options={'ordering': ('uri',), 'verbose_name': 'Attribute', 'verbose_name_plural': 'Attributes'},
        ),
        migrations.AlterField(
            model_name='attribute',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, help_text='Parent attribute in the domain model.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='domain.Attribute', verbose_name='Parent attribute'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-06 12:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0013_meta'),
        ('questions', '0018_data_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='questionentity',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='List of conditions evaluated for this question/questionset.', to='conditions.Condition', verbose_name='Conditions'),
        ),
    ]



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Task
from ..serializers.v1 import TaskSerializer
from ..validators import TaskUniqueURIValidator


def test_unique_uri_validator_create(db):
    TaskUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_create_error(db):
    with pytest.raises(ValidationError):
        TaskUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': Task.objects.last().key
        })


def test_unique_uri_validator_update(db):
    task = Task.objects.first()

    TaskUniqueURIValidator(task)({
        'uri_prefix': task.uri_prefix,
        'key': task.key
    })


def test_unique_uri_validator_update_error(db):
    task = Task.objects.first()

    with pytest.raises(ValidationError):
        TaskUniqueURIValidator(task)({
            'uri_prefix': task.uri_prefix,
            'key': Task.objects.last().key
        })


def test_unique_uri_validator_serializer_create(db):
    validator = TaskUniqueURIValidator()
    validator.set_context(TaskSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_serializer_create_error(db):
    validator = TaskUniqueURIValidator()
    validator.set_context(TaskSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': Task.objects.last().key
        })


def test_unique_uri_validator_serializer_update(db):
    task = Task.objects.first()

    validator = TaskUniqueURIValidator()
    validator.set_context(TaskSerializer(instance=task))

    validator({
        'uri_prefix': task.uri_prefix,
        'key': task.key
    })


def test_unique_uri_validator_serializer_update_error(db):
    task = Task.objects.first()

    validator = TaskUniqueURIValidator()
    validator.set_context(TaskSerializer(instance=task))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': task.uri_prefix,
            'key': Task.objects.last().key
        })



from django.conf import settings
from django.db import models
from mptt.models import TreeManager
from mptt.querysets import TreeQuerySet

from rdmo.accounts.utils import is_site_manager
from rdmo.core.managers import CurrentSiteManagerMixin


class ProjectQuerySet(TreeQuerySet):

    def filter_current_site(self):
        return self.filter(site=settings.SITE_ID)

    def filter_user(self, user):
        if user.is_authenticated:
            if user.has_perm('projects.view_project'):
                return self.all()
            elif is_site_manager(user):
                return self.filter_current_site()
            else:
                queryset = self.filter(user=user)
                for instance in queryset:
                    queryset |= instance.get_descendants()
                return queryset.distinct()
        else:
            return self.none()


class MembershipQuerySet(models.QuerySet):

    def filter_current_site(self):
        return self.filter(project__site=settings.SITE_ID)

    def filter_user(self, user):
        if user.is_authenticated:
            if user.has_perm('projects.view_membership'):
                return self.all()
            elif is_site_manager(user):
                return self.filter_current_site()
            else:
                from .models import Project
                projects = Project.objects.filter_user(user)
                return self.filter(project__in=projects)
        else:
            return self.objects.none()


class IssueQuerySet(models.QuerySet):

    def filter_current_site(self):
        return self.filter(project__site=settings.SITE_ID)

    def filter_user(self, user):
        if user.is_authenticated:
            if user.has_perm('projects.view_integration'):
                return self.all()
            elif is_site_manager(user):
                return self.filter_current_site()
            else:
                from .models import Project
                projects = Project.objects.filter_user(user)
                return self.filter(project__in=projects)
        else:
            return self.none()


class IntegrationQuerySet(models.QuerySet):

    def filter_current_site(self):
        return self.filter(project__site=settings.SITE_ID)

    def filter_user(self, user):
        if user.is_authenticated:
            if user.has_perm('projects.view_issue'):
                return self.all()
            elif is_site_manager(user):
                return self.filter_current_site()
            else:
                from .models import Project
                projects = Project.objects.filter_user(user)
                return self.filter(project__in=projects)
        else:
            return self.none()


class SnapshotQuerySet(models.QuerySet):

    def filter_current_site(self):
        return self.filter(project__site=settings.SITE_ID)

    def filter_user(self, user):
        if user.is_authenticated:
            if user.has_perm('projects.view_snapshot'):
                return self.all()
            elif is_site_manager(user):
                return self.filter_current_site()
            else:
                from .models import Project
                projects = Project.objects.filter_user(user)
                return self.filter(project__in=projects)
        else:
            return self.none()


class ValueQuerySet(models.QuerySet):

    def filter_current_site(self):
        return self.filter(project__site=settings.SITE_ID)

    def filter_user(self, user):
        if user.is_authenticated:
            if user.has_perm('projects.view_value'):
                return self.all()
            elif is_site_manager(user):
                return self.filter_current_site()
            else:
                from .models import Project
                projects = Project.objects.filter_user(user)
                return self.filter(project__in=projects)
        else:
            return self.none()


class ProjectManager(CurrentSiteManagerMixin, TreeManager):

    def get_queryset(self):
        return ProjectQuerySet(self.model, using=self._db)

    def filter_user(self, user):
        return self.get_queryset().filter_user(user)


class MembershipManager(CurrentSiteManagerMixin, models.Manager):

    def get_queryset(self):
        return MembershipQuerySet(self.model, using=self._db)

    def filter_user(self, user):
        return self.get_queryset().filter_user(user)


class IssueManager(CurrentSiteManagerMixin, models.Manager):

    def get_queryset(self):
        return IssueQuerySet(self.model, using=self._db)

    def filter_user(self, user):
        return self.get_queryset().filter_user(user)

    def active(self):
        return self.get_queryset().active()


class IntegrationManager(CurrentSiteManagerMixin, models.Manager):

    def get_queryset(self):
        return IntegrationQuerySet(self.model, using=self._db)

    def filter_user(self, user):
        return self.get_queryset().filter_user(user)


class SnapshotManager(CurrentSiteManagerMixin, models.Manager):

    def get_queryset(self):
        return SnapshotQuerySet(self.model, using=self._db)

    def filter_user(self, user):
        return self.get_queryset().filter_user(user)


class ValueManager(CurrentSiteManagerMixin, models.Manager):

    def get_queryset(self):
        return ValueQuerySet(self.model, using=self._db)

    def filter_user(self, user):
        return self.get_queryset().filter_user(user)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-28 11:23
from __future__ import unicode_literals

from django.db import migrations


def set_question_unit_and_type(apps, schema_editor):
    Question = apps.get_model('questions', 'Question')

    for question in Question.objects.all():
        try:
            question.value_type = question.attribute_entity.attribute.value_type
        except AttributeError:
            question.value_type = 'text'

        try:
            question.unit = question.attribute_entity.attribute.unit or ''
        except AttributeError:
            question.unit = ''

        question.save()


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0015_question_unit_and_type'),
    ]

    operations = [
        migrations.RunPython(set_question_unit_and_type),
    ]



from django import forms
from django.contrib import admin

from .models import Condition
from .validators import ConditionLockedValidator, ConditionUniqueURIValidator


class ConditionAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = Condition
        fields = '__all__'

    def clean(self):
        ConditionUniqueURIValidator(self.instance)(self.cleaned_data)
        ConditionLockedValidator(self.instance)(self.cleaned_data)


class ConditionAdmin(admin.ModelAdmin):
    form = ConditionAdminForm

    search_fields = ('uri', 'source__uri')
    list_display = ('uri', 'source', 'relation', 'target_text', 'target_option')
    readonly_fields = ('uri', )
    list_filter = ('relation', )


admin.site.register(Condition, ConditionAdmin)



import logging

from django.conf import settings
from django.http import Http404
from django.template import TemplateSyntaxError
from django.views.generic import DetailView
from rdmo.core.constants import VALUE_TYPE_FILE
from rdmo.core.utils import render_to_format
from rdmo.core.views import ObjectPermissionMixin
from rdmo.views.models import View

from ..models import Project, Snapshot
from ..utils import get_value_path

logger = logging.getLogger(__name__)


class ProjectViewView(ObjectPermissionMixin, DetailView):
    model = Project
    queryset = Project.objects.all()
    permission_required = 'projects.view_project_object'
    template_name = 'projects/project_view.html'

    def get_context_data(self, **kwargs):
        context = super(ProjectViewView, self).get_context_data(**kwargs)

        try:
            context['current_snapshot'] = context['project'].snapshots.get(pk=self.kwargs.get('snapshot_id'))
        except Snapshot.DoesNotExist:
            context['current_snapshot'] = None

        try:
            context['view'] = context['project'].views.get(pk=self.kwargs.get('view_id'))
        except View.DoesNotExist:
            raise Http404

        try:
            context['rendered_view'] = context['view'].render(context['project'],
                                                              snapshot=context['current_snapshot'])
        except TemplateSyntaxError:
            context['rendered_view'] = None

        # collect values with files, remove double files and order them.
        context['attachments'] = context['project'].values.filter(snapshot=context['current_snapshot']) \
                                                          .filter(value_type=VALUE_TYPE_FILE) \
                                                          .order_by('file')

        context.update({
            'snapshots': list(context['project'].snapshots.values('id', 'title')),
            'export_formats': settings.EXPORT_FORMATS
        })

        return context


class ProjectViewExportView(ObjectPermissionMixin, DetailView):
    model = Project
    queryset = Project.objects.all()
    permission_required = 'projects.view_project_object'

    def get_context_data(self, **kwargs):
        export_format = self.kwargs.get('format')

        context = super(ProjectViewExportView, self).get_context_data(**kwargs)

        try:
            context['current_snapshot'] = context['project'].snapshots.get(pk=self.kwargs.get('snapshot_id'))
        except Snapshot.DoesNotExist:
            context['current_snapshot'] = None

        try:
            context['view'] = context['project'].views.get(pk=self.kwargs.get('view_id'))
        except View.DoesNotExist:
            raise Http404

        try:
            context['rendered_view'] = context['view'].render(context['project'],
                                                              snapshot=context['current_snapshot'],
                                                              export_format=export_format)
        except TemplateSyntaxError:
            context['rendered_view'] = None

        context.update({
            'title': context['project'].title,
            'format': export_format,
            'resource_path': get_value_path(context['project'], context['current_snapshot'])
        })

        return context

    def render_to_response(self, context, **response_kwargs):
        return render_to_format(self.request, context['format'], context['title'], 'projects/project_view_export.html', context)



# Generated by Django 2.2.24 on 2021-07-23 15:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0063_questionset_questionset'),
    ]

    operations = [
        migrations.AlterField(
            model_name='question',
            name='widget_type',
            field=models.CharField(help_text='Type of widget for this question.', max_length=16, verbose_name='Widget type'),
        ),
    ]



from .continuation import Continuation
from .integration import Integration, IntegrationOption
from .invite import Invite
from .issue import Issue, IssueResource
from .membership import Membership
from .project import Project
from .snapshot import Snapshot
from .value import Value



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-11 13:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='view',
            name='template',
            field=models.TextField(blank=True, null=True),
        ),
    ]



import logging

from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render

from rdmo.core.utils import get_next, get_referer_path_info

from .models import Overlay

logger = logging.getLogger(__name__)


@login_required()
def reset_overlays(request):
    if request.method == 'POST':
        Overlay.objects.filter(user=request.user).delete()
        return redirect(get_next(request))

    return render(request, 'overlays/reset_overlays.html', {
        'next': get_referer_path_info(request, default='/')
    })



# Generated by Django 2.2.7 on 2019-11-05 14:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0021_view_catalogs'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='view',
            managers=[
            ],
        ),
    ]



import base64
import io
import logging
import mimetypes

from django.core.files import File

from rdmo.core.plugins import Plugin
from rdmo.core.xml import get_ns_map, get_uri, read_xml_file
from rdmo.domain.models import Attribute
from rdmo.options.models import Option
from rdmo.questions.models import Catalog
from rdmo.tasks.models import Task
from rdmo.views.models import View

from .models import Project, Snapshot, Value

log = logging.getLogger(__name__)


class Import(Plugin):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.file_name = None
        self.current_project = None
        self.project = None
        self.catalog = None

        self._attributes = {attribute.uri: attribute for attribute in Attribute.objects.all()}
        self._options = {option.uri: option for option in Option.objects.all()}
        self._tasks = {task.uri: task for task in Task.objects.all()}
        self._views = {view.uri: view for view in View.objects.all()}

        self.values = []
        self.snapshots = []
        self.tasks = []
        self.views = []

    def check(self):
        raise NotImplementedError

    def process(self):
        raise NotImplementedError

    def get_attribute(self, attribute_uri):
        try:
            return self._attributes.get(attribute_uri)
        except KeyError:
            log.info('Attribute %s not in db. Skipping.', attribute_uri)

    def get_option(self, option_uri):
        try:
            return self._options.get(option_uri)
        except KeyError:
            log.info('Option %s not in db. Skipping.', option_uri)

    def get_task(self, tasks_uri):
        try:
            return self._tasks.get(tasks_uri)
        except KeyError:
            log.info('Task %s not in db. Skipping.', tasks_uri)

    def get_view(self, view_uri):
        try:
            return self._views.get(view_uri)
        except KeyError:
            log.info('View %s not in db. Skipping.', view_uri)


class RDMOXMLImport(Import):

    def check(self):
        file_type, encoding = mimetypes.guess_type(self.file_name)
        if file_type == 'application/xml' or file_type == 'text/xml':
            self.root = read_xml_file(self.file_name)
            if self.root and self.root.tag == 'project':
                self.ns_map = get_ns_map(self.root)
                return True

    def process(self):
        if self.current_project is None:
            catalog_uri = get_uri(self.root.find('catalog'), self.ns_map)
            try:
                self.catalog = Catalog.objects.all().get(uri=catalog_uri)
            except Catalog.DoesNotExist:
                log.info('Catalog not in db. Created with uri %s', catalog_uri)
                self.catalog = Catalog.objects.all().first()

            self.project = Project()
            self.project.title = self.root.find('title').text or ''
            self.project.description = self.root.find('description').text or ''
            self.project.created = self.root.find('created').text
            self.project.catalog = self.catalog
        else:
            self.catalog = self.current_project.catalog

        tasks_node = self.root.find('tasks')
        if tasks_node is not None:
            for task_node in tasks_node.findall('task'):
                task_uri = get_uri(task_node, self.ns_map)
                if task_uri:
                    task = self.get_task(task_uri)
                    if task:
                        self.tasks.append(task)

        views_node = self.root.find('views')
        if views_node is not None:
            for view_node in views_node.findall('view'):
                view_uri = get_uri(view_node, self.ns_map)
                if view_uri:
                    view = self.get_view(view_uri)
                    if view:
                        self.views.append(view)

        values_node = self.root.find('values')
        if values_node is not None:
            for value_node in values_node.findall('value'):
                self.values.append(self.get_value(value_node))

        snapshots_node = self.root.find('snapshots')
        if snapshots_node is not None:
            for snapshot_index, snapshot_node in enumerate(snapshots_node.findall('snapshot')):
                if snapshot_node is not None:
                    snapshot = Snapshot()
                    snapshot.title = snapshot_node.find('title').text or ''
                    snapshot.description = snapshot_node.find('description').text or ''
                    snapshot.created = snapshot_node.find('created').text

                    snapshot_values = []
                    snapshot_values_node = snapshot_node.find('values')
                    if snapshot_values_node is not None:
                        for snapshot_value_node in snapshot_values_node.findall('value'):
                            snapshot_values.append(self.get_value(snapshot_value_node))

                    snapshot.snapshot_index = snapshot_index
                    snapshot.snapshot_values = snapshot_values

                    self.snapshots.append(snapshot)

    def get_value(self, value_node):
        value = Value()

        attribute_uri = get_uri(value_node.find('attribute'), self.ns_map)
        if attribute_uri is not None:
            value.attribute = self.get_attribute(attribute_uri)

        try:
            value.set_prefix = value_node.find('set_prefix').text or ''
        except AttributeError:
            value.set_prefix = ''

        value.set_index = int(value_node.find('set_index').text)
        value.collection_index = int(value_node.find('collection_index').text)

        value.text = value_node.find('text').text or ''

        option_uri = get_uri(value_node.find('option'), self.ns_map)
        if option_uri:
            value.option = self.get_option(option_uri)

        value.file_import = None
        file_node = value_node.find('file')
        if file_node is not None:
            file_string = file_node.text
            if file_string is not None:
                value.file_import = {
                    'name': file_node.attrib.get('name', 'file.dat'),
                    'file': File(io.BytesIO(base64.b64decode(file_string)))
                }

        value_type_node = value_node.find('value_type')
        if value_type_node is not None:
            value.value_type = value_type_node.text or ''

        unit_node = value_node.find('unit')
        if unit_node is not None:
            value.unit = unit_node.text or ''

        external_id_node = value_node.find('external_id')
        if external_id_node is not None:
            value.external_id = external_id_node.text or ''

        value.created = value_node.find('created').text
        value.updated = value_node.find('updated').text

        return value



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-29 15:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0015_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='timeframe',
            name='end_attribute',
        ),
        migrations.RemoveField(
            model_name='timeframe',
            name='start_attribute',
        ),
        migrations.RemoveField(
            model_name='timeframe',
            name='task',
        ),
        migrations.DeleteModel(
            name='TimeFrame',
        ),
    ]



# Generated by Django 2.2.2 on 2019-07-12 09:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0027_project_site'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='site',
            field=models.ForeignKey(help_text='The site this project belongs to (in a multi site setup).', on_delete=django.db.models.deletion.CASCADE, to='sites.Site', verbose_name='Site'),
        ),
    ]



import logging

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.template import TemplateSyntaxError
from django.template.loader import render_to_string
from django.views.generic import DetailView, UpdateView
from rest_framework.reverse import reverse

from rdmo.core.mail import send_mail
from rdmo.core.utils import render_to_format
from rdmo.core.views import ObjectPermissionMixin, RedirectViewMixin
from rdmo.views.utils import ProjectWrapper

from ..forms import IssueMailForm, IssueSendForm
from ..models import Issue
from ..utils import get_value_path

logger = logging.getLogger(__name__)


class IssueDetailView(ObjectPermissionMixin, DetailView):
    permission_required = 'projects.view_issue_object'

    def get_queryset(self):
        return Issue.objects.filter(project_id=self.kwargs.get('project_id'))

    def get_permission_object(self):
        return self.get_object().project

    def get_context_data(self, **kwargs):
        project = self.get_object().project
        conditions = self.get_object().task.conditions.all()

        sources = []
        for condition in conditions:
            sources.append({
                'source': condition.source,
                'questions': condition.source.questions.filter(questionset__section__catalog=project.catalog),
                'values': condition.source.values.filter(project=project, snapshot=None)
            })

        kwargs['project'] = project
        kwargs['sources'] = sources
        return super().get_context_data(**kwargs)


class IssueUpdateView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    fields = ('status', )
    permission_required = 'projects.change_issue_object'

    def get_queryset(self):
        return Issue.objects.filter(project_id=self.kwargs.get('project_id'))

    def get_permission_object(self):
        return self.get_object().project


class IssueSendView(ObjectPermissionMixin, RedirectViewMixin, DetailView):
    permission_required = 'projects.change_issue_object'
    template_name = 'projects/issue_send.html'

    def dispatch(self, *args, **kwargs):
        if not settings.PROJECT_SEND_ISSUE:
            raise Http404

        return super().dispatch(*args, **kwargs)

    def get_queryset(self):
        return Issue.objects.filter(project_id=self.kwargs.get('project_id'))

    def get_permission_object(self):
        return self.get_object().project

    def get_success_url(self):
        if self.request.GET.get('next'):
            return self.request.GET.get('next')
        else:
            return self.get_object().project.get_absolute_url()

    def get_context_data(self, **kwargs):
        self.object = self.get_object()

        issue = self.object
        project = self.get_object().project
        project_url = self.request.build_absolute_uri(project.get_absolute_url())
        site = Site.objects.get_current()
        site_url = self.request.build_absolute_uri(reverse('home'))

        if 'form' not in kwargs:
            template_context = {
                'issue': issue,
                'project': project,
                'project_url': project_url,
                'site': site,
                'site_url': site_url,
                'user': self.request.user
            }
            subject = render_to_string('projects/issue_send_subject.txt', template_context, request=self.request)
            message = render_to_string('projects/issue_send_message.txt', template_context, request=self.request)

            kwargs['form'] = IssueSendForm(initial={
                'subject': subject,
                'message': message
            }, project=project)

        if 'mail_form' not in kwargs:
            kwargs['mail_form'] = IssueMailForm()

        context = super().get_context_data(**kwargs)
        context['views'] = project.views.all()
        context['integrations'] = project.integrations.all()
        return context

    def post(self, request, *args, **kwargs):
        issue = self.get_object()
        project = issue.project

        form = IssueSendForm(request.POST, project=project)
        mail_form = IssueMailForm(request.POST)

        if form.is_valid():
            subject = form.cleaned_data.get('subject')
            message = form.cleaned_data.get('message')

            snapshot = project.snapshots.filter(pk=form.cleaned_data.get('snapshot')).first()
            attachments_format = form.cleaned_data.get('attachments_format')

            attachments = []
            if form.cleaned_data.get('attachments_answers'):
                title = '{}.{}'.format(project.title, attachments_format)
                response = self.render_project_answers(project, snapshot, attachments_format)
                attachments.append((title, response.content, response['content-type']))

            for view in form.cleaned_data.get('attachments_views'):
                title = '{}.{}'.format(project.title, attachments_format)
                response = self.render_project_views(project, snapshot, view, attachments_format)
                attachments.append((title, response.content, response['content-type']))

            for value in form.cleaned_data.get('attachments_files'):
                attachments.append((value.file_name, value.file.read(), value.file_type))

            integration_id = request.POST.get('integration')
            if integration_id:
                # send via integration
                try:
                    integration = project.integrations.get(pk=integration_id)
                    return integration.provider.send_issue(request, issue, integration, subject, message, attachments)
                except (ObjectDoesNotExist, AttributeError):
                    pass
            else:
                if mail_form.is_valid():
                    to_emails = mail_form.cleaned_data.get('recipients', []) + mail_form.cleaned_data.get('recipients_input', [])
                    cc_emails = [request.user.email]
                    reply_to = [request.user.email]

                    # send the email
                    send_mail(subject, message, to=to_emails, cc=cc_emails, reply_to=reply_to, attachments=attachments)

                    # update issue status
                    issue.status = Issue.ISSUE_STATUS_IN_PROGRESS
                    issue.save()

                    return HttpResponseRedirect(self.get_success_url())

        return self.render_to_response(self.get_context_data(form=form, mail_form=mail_form))

    def render_project_answers(self, project, snapshot, attachments_format):
        return render_to_format(self.request, attachments_format, project.title, 'projects/project_answers_export.html', {
            'project': ProjectWrapper(project, snapshot),
            'format': attachments_format,
            'title': project.title,
            'resource_path': get_value_path(project, snapshot)
        })

    def render_project_views(self, project, snapshot, view, attachments_format):
        try:
            rendered_view = view.render(project, snapshot)
        except TemplateSyntaxError:
            return HttpResponse()

        return render_to_format(self.request, attachments_format, project.title, 'projects/project_view_export.html', {
            'format': attachments_format,
            'title': project.title,
            'view': view,
            'rendered_view': rendered_view,
            'resource_path': get_value_path(project, snapshot)
        })



# Generated by Django 2.2.7 on 2019-11-05 10:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0047_manager'),
    ]

    operations = [
        migrations.AddField(
            model_name='catalog',
            name='help_lang1',
            field=models.TextField(blank=True, help_text='The help text for this catalog in the primary language.', verbose_name='Help (primary)'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='help_lang2',
            field=models.TextField(blank=True, help_text='The help text for this catalog in the secondary language.', verbose_name='Help (secondary)'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='help_lang3',
            field=models.TextField(blank=True, help_text='The help text for this catalog in the tertiary language.', verbose_name='Help (tertiary)'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='help_lang4',
            field=models.TextField(blank=True, help_text='The help text for this catalog in the quaternary language.', verbose_name='Help (quaternary)'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='help_lang5',
            field=models.TextField(blank=True, help_text='The help text for this catalog in the quinary language.', verbose_name='Help (quinary)'),
        ),
    ]



import logging

from django.core.exceptions import ObjectDoesNotExist
from rest_framework.permissions import (DjangoModelPermissions,
                                        DjangoObjectPermissions)

logger = logging.getLogger(__name__)


def log_result(func):
    '''
    A decorator to automatically log the arguments and the results of calls
    to has_permission and has_object_permission.
    '''
    def wrapper(self, request, *args):
        result = func(self, request, *args)

        class_name = self.__class__.__name__
        func_name = func.__name__
        logger.debug('%s.%s path=%s method=%s user=%s result=%s',
                     class_name, func_name, request.path, request.method, request.user, result)
        return result

    return wrapper


class HasModelPermission(DjangoModelPermissions):

    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s'],
        'OPTIONS': ['%(app_label)s.view_%(model_name)s'],
        'HEAD': ['%(app_label)s.view_%(model_name)s'],
        'POST': ['%(app_label)s.add_%(model_name)s'],
        'PUT': ['%(app_label)s.change_%(model_name)s'],
        'PATCH': ['%(app_label)s.change_%(model_name)s'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s'],
    }

    @log_result
    def has_permission(self, request, view):
        return super().has_permission(request, view)

    @log_result
    def has_object_permission(self, request, view, obj):
        if not (request.user and request.user.is_authenticated):
            return False

        # has_object_permission needs to follow has_permission
        return self.has_permission(request, view)


class HasObjectPermission(DjangoObjectPermissions):

    perms_map = {
        'GET': ['%(app_label)s.view_%(model_name)s_object'],
        'OPTIONS': ['%(app_label)s.view_%(model_name)s_object'],
        'HEAD': ['%(app_label)s.view_%(model_name)s_object'],
        'POST': ['%(app_label)s.add_%(model_name)s_object'],
        'PUT': ['%(app_label)s.change_%(model_name)s_object'],
        'PATCH': ['%(app_label)s.change_%(model_name)s_object'],
        'DELETE': ['%(app_label)s.delete_%(model_name)s_object'],
    }

    @log_result
    def has_permission(self, request, view):
        if not (request.user and request.user.is_authenticated):
            return False

        # check if this is a detail view (retrieve, update, partial_update, destroy) or not (list, create)
        if view.detail:
            # for retrieve, update, partial_update, or destroy return True
            # the permission will be checked on object level (in the next step)
            return True
        else:
            try:
                # for list or create we need to check the permission object from the view
                # and check that the user has the correct permission on this object
                permission_object = view.get_list_permission_object()
                return super().has_object_permission(request, view, permission_object)
            except ObjectDoesNotExist:
                # return False if the database query fails
                return False
            except AttributeError:
                # return True if the function is not defined in the view
                # the permission will be checked on object level (in the next step)
                return True

    @log_result
    def has_object_permission(self, request, view, obj):
        # get the permission object from the view
        try:
            permission_object = view.get_detail_permission_object(obj)
            return super().has_object_permission(request, view, permission_object)
        except ObjectDoesNotExist:
            # return False if the database query fails
            return False
        except AttributeError:
            # just take the input obj if the function is not defined in the view
            return super().has_object_permission(request, view, obj)



# Generated by Django 2.2.2 on 2019-07-01 15:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
        ('views', '0018_view_sites'),
    ]

    operations = [
        migrations.AddField(
            model_name='view',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups for which this view is active.', to='auth.Group', verbose_name='Group'),
        ),
    ]



import pytest
from django.urls import reverse

from ..models import Project

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('api', 'api'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5, 7],
    'author': [1, 3, 5, 8],
    'guest': [1, 3, 5, 9],
    'api': [1, 2, 3, 4, 5, 6, 7, 8, 9],
    'site': [1, 2, 3, 4, 5, 6, 7, 8, 9]
}

change_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5, 7],
    'api': [1, 2, 3, 4, 5, 6, 7, 8, 9],
    'site': [1, 2, 3, 4, 5, 6, 7, 8, 9]
}

delete_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'api': [1, 2, 3, 4, 5, 6, 7, 8, 9],
    'site': [1, 2, 3, 4, 5, 6, 7, 8, 9]
}

urlnames = {
    'list': 'v1-projects:project-list',
    'detail': 'v1-projects:project-detail',
    'overview': 'v1-projects:project-overview',
    'resolve': 'v1-projects:project-resolve',
    'progress': 'v1-projects:project-progress'
}

projects = [1, 2, 3, 4, 5]
conditions = [1]

project_values = 37
project_total = 54
catalog_id = 1


@pytest.mark.parametrize('username,password', users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)

    if password:
        assert response.status_code == 200
        assert isinstance(response.json(), list)

        if username == 'user':
            assert sorted([item['id'] for item in response.json()]) == []
        else:
            values_list = Project.objects.filter(id__in=view_project_permission_map.get(username, [])) \
                                         .order_by('id').values_list('id', flat=True)
            assert sorted([item['id'] for item in response.json()]) == list(values_list)
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_detail(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[project_id])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') == project_id
    else:
        if password:
            assert response.status_code == 404
        else:
            assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    data = {
        'title': 'Lorem ipsum dolor sit amet',
        'description': 'At vero eos et accusam et justo duo dolores et ea rebum.',
        'catalog': catalog_id
    }
    response = client.post(url, data)

    if password:
        assert response.status_code == 201
        assert isinstance(response.json(), dict)
        assert Project.objects.get(id=response.json().get('id'))
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_create_parent(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    data = {
        'title': 'Lorem ipsum dolor sit amet',
        'description': 'At vero eos et accusam et justo duo dolores et ea rebum.',
        'catalog': catalog_id,
        'parent': project_id
    }
    response = client.post(url, data)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 201
        assert isinstance(response.json(), dict)
        assert Project.objects.get(id=response.json().get('id'))
    else:
        if password:
            assert response.status_code == 400
        else:
            assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_update(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse(urlnames['detail'], args=[project_id])
    data = {
        'title': 'New title',
        'description': project.description,
        'catalog': project.catalog.id
    }
    response = client.put(url, data, content_type='application/json')

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 200
        assert Project.objects.get(id=project_id).title == 'New title'
        assert Project.objects.get(id=project_id).description == project.description
    else:
        if project_id in view_project_permission_map.get(username, []):
            assert response.status_code == 403
        elif password:
            assert response.status_code == 404
        else:
            assert response.status_code == 401

        assert Project.objects.get(id=project_id).title == project.title
        assert Project.objects.get(id=project_id).description == project.description


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_delete(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[project_id])
    response = client.delete(url)

    if project_id in delete_project_permission_map.get(username, []):
        assert response.status_code == 204
        assert Project.objects.filter(id=project_id).first() is None
    else:
        if project_id in view_project_permission_map.get(username, []):
            assert response.status_code == 403
        elif password:
            assert response.status_code == 404
        else:
            assert response.status_code == 401

        assert Project.objects.filter(id=project_id).first()


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('condition_id', conditions)
def test_overview(db, client, username, password, project_id, condition_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['overview'], args=[project_id])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') == project_id
    else:
        if password:
            assert response.status_code == 404
        else:
            assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('condition_id', conditions)
def test_resolve(db, client, username, password, project_id, condition_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['resolve'], args=[project_id]) + '?condition={}'.format(condition_id)
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
    else:
        if password:
            assert response.status_code == 404
        else:
            assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_progress(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['progress'], args=[project_id])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)

        if project_id == 1:
            assert response.json().get('values') == project_values
        else:
            assert response.json().get('values') == 1

        assert response.json().get('total') == project_total
    else:
        if password:
            assert response.status_code == 404
        else:
            assert response.status_code == 401



from allauth.account.adapter import DefaultAccountAdapter
from allauth.socialaccount.adapter import DefaultSocialAccountAdapter
from django.conf import settings


class AccountAdapter(DefaultAccountAdapter):

    def is_open_for_signup(self, request):
        return settings.ACCOUNT_SIGNUP


class SocialAccountAdapter(DefaultSocialAccountAdapter):

    def is_open_for_signup(self, request, sociallogin):
        return settings.SOCIALACCOUNT_SIGNUP



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0001_initial_after_reset'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('questions', '0001_initial_after_reset'),
    ]

    operations = [
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(verbose_name='created', editable=False)),
                ('updated', models.DateTimeField(verbose_name='updated', editable=False)),
                ('title', models.CharField(max_length=256)),
                ('description', models.TextField(help_text='You can use markdown syntax in the description.', blank=True)),
                ('catalog', models.ForeignKey(related_name='+', to='questions.Catalog', help_text='The catalog which will be used for this project.', on_delete=models.SET_NULL)),
            ],
            options={
                'ordering': ('title',),
                'verbose_name': 'Project',
                'verbose_name_plural': 'Projects',
            },
        ),
        migrations.CreateModel(
            name='Snapshot',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(verbose_name='created', editable=False)),
                ('updated', models.DateTimeField(verbose_name='updated', editable=False)),
                ('project', models.ForeignKey(related_name='snapshots', to='projects.Project', on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ('project', 'pk'),
                'verbose_name': 'Snapshot',
                'verbose_name_plural': 'Snapshots',
            },
        ),
        migrations.CreateModel(
            name='Value',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(verbose_name='created', editable=False)),
                ('updated', models.DateTimeField(verbose_name='updated', editable=False)),
                ('set_index', models.IntegerField(default=0)),
                ('collection_index', models.IntegerField(default=0)),
                ('text', models.TextField(null=True, blank=True)),
                ('attribute', models.ForeignKey(related_name='values', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='domain.Attribute', null=True)),
                ('option', models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='domain.Option', null=True)),
                ('snapshot', models.ForeignKey(related_name='values', to='projects.Snapshot', on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name': 'Value',
                'verbose_name_plural': 'Values',
            },
        ),
        migrations.AddField(
            model_name='project',
            name='current_snapshot',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='projects.Snapshot', null=True),
        ),
        migrations.AddField(
            model_name='project',
            name='owner',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-19 16:53
from __future__ import unicode_literals

from django.db import migrations
from django.utils.text import slugify


def create_identifier(apps, schema_editor):

    Catalog = apps.get_model('questions', 'Catalog')
    Section = apps.get_model('questions', 'Section')
    Subsection = apps.get_model('questions', 'Subsection')
    QuestionEntity = apps.get_model('questions', 'QuestionEntity')

    for obj in Catalog.objects.all():
        if not obj.key:
            obj.key = slugify(obj.title_en)
        obj.save()

    for obj in Section.objects.all():
        if not obj.key:
            obj.key = slugify(obj.title_en)
        obj.save()

    for obj in Subsection.objects.all():
        if not obj.key:
            obj.key = slugify(obj.title_en)
        obj.save()

    for obj in QuestionEntity.objects.all():
        if not obj.key:
            obj.key = slugify(obj.attribute_entity.key)
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0007_refactoring'),
    ]

    operations = [
        migrations.RunPython(create_identifier),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-21 10:28
from __future__ import unicode_literals

import rdmo.core.models
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('domain', '0019_meta'),
    ]

    operations = [
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256, validators=[django.core.validators.RegexValidator('^[a-zA-z0-9_]*$', 'Only letters, numbers, or underscores are allowed.')])),
                ('order', models.IntegerField(null=True)),
                ('text_en', models.CharField(max_length=256)),
                ('text_de', models.CharField(max_length=256)),
                ('additional_input', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ('optionset', 'order'),
                'verbose_name': 'Option',
                'verbose_name_plural': 'Options',
            },
            bases=(models.Model, rdmo.core.models.TranslationMixin),
        ),
        migrations.CreateModel(
            name='OptionSet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256, validators=[django.core.validators.RegexValidator('^[a-zA-z0-9_]*$', 'Only letters, numbers, or underscores are allowed.')])),
                ('order', models.IntegerField(null=True)),
            ],
            options={
                'ordering': ('title', ),
                'verbose_name': 'OptionSet',
                'verbose_name_plural': 'OptionSets',
            },
        ),
        migrations.AddField(
            model_name='option',
            name='optionset',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='options', to='options.OptionSet'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-07-01 16:00

from django.db import migrations
import django.db.models.manager
import rdmo.questions.managers


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0046_catalog_groups'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='catalog',
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', rdmo.questions.managers.CatalogManager()),
            ],
        ),
    ]



import pytest
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Option, OptionSet
from ..serializers.v1 import OptionSerializer
from ..validators import OptionLockedValidator


def test_create(db):
    OptionLockedValidator()({
        'optionset': OptionSet.objects.first(),
        'locked': False
    })


def test_create_locked(db):
    OptionLockedValidator()({
        'optionset': OptionSet.objects.first(),
        'locked': True
    })


def test_update(db):
    option = Option.objects.first()

    OptionLockedValidator(option)({
        'optionset': option.optionset,
        'locked': False
    })


def test_update_error(db):
    option = Option.objects.first()
    option.locked = True
    option.save()

    with pytest.raises(ValidationError):
        OptionLockedValidator(option)({
            'optionset': option.optionset,
            'locked': True
        })


def test_update_parent_error(db):
    option = Option.objects.first()
    option.optionset.locked = True
    option.optionset.save()

    with pytest.raises(ValidationError):
        OptionLockedValidator(option)({
            'optionset': option.optionset,
            'locked': False
        })


def test_update_lock(db):
    option = Option.objects.first()

    OptionLockedValidator(option)({
        'optionset': option.optionset,
        'locked': True
    })


def test_update_unlock(db):
    option = Option.objects.first()
    option.locked = True
    option.save()

    OptionLockedValidator(option)({
        'optionset': option.optionset,
        'locked': False
    })


def test_serializer_create(db):
    validator = OptionLockedValidator()
    validator.set_context(OptionSerializer())

    validator({
        'optionset': OptionSet.objects.first(),
        'locked': False
    })


def test_serializer_create_locked(db):
    validator = OptionLockedValidator()
    validator.set_context(OptionSerializer())

    validator({
        'optionset': OptionSet.objects.first(),
        'locked': True
    })


def test_serializer_update(db):
    option = Option.objects.first()

    validator = OptionLockedValidator()
    validator.set_context(OptionSerializer(instance=option))

    validator({
        'optionset': option.optionset,
        'locked': False
    })


def test_serializer_update_error(db):
    option = Option.objects.first()
    option.locked = True
    option.save()

    validator = OptionLockedValidator()
    validator.set_context(OptionSerializer(instance=option))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'optionset': option.optionset,
            'locked': True
        })


def test_serializer_update_parent_error(db):
    option = Option.objects.first()
    option.optionset.locked = True
    option.optionset.save()

    validator = OptionLockedValidator()
    validator.set_context(OptionSerializer(instance=option))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'optionset': option.optionset,
            'locked': True
        })


def test_serializer_update_lock(db):
    option = Option.objects.first()

    validator = OptionLockedValidator()
    validator.set_context(OptionSerializer(instance=option))

    validator({
        'optionset': option.optionset,
        'locked': True
    })


def test_serializer_update_unlock(db):
    option = Option.objects.first()
    option.locked = True
    option.save()

    validator = OptionLockedValidator()
    validator.set_context(OptionSerializer(instance=option))

    validator({
        'optionset': option.optionset,
        'locked': False
    })



import pytest
from django.core.exceptions import ValidationError
from rest_framework import serializers

from ..validators import InstanceValidator


def test_instance_validator():
    validator = InstanceValidator()
    assert validator.instance is None
    assert validator.serializer is None


def test_instance_validator_instance():
    instance = object()
    validator = InstanceValidator(instance)
    assert validator.instance is instance
    assert validator.serializer is None


def test_instance_validator_serializer():
    validator = InstanceValidator()
    serializer = serializers.Serializer()
    validator.set_context(serializer)

    assert validator.serializer == serializer
    assert validator.serializer.instance is None


def test_instance_validator_serializer_instance():
    validator = InstanceValidator()
    instance = object()
    serializer = serializers.Serializer(instance=instance)
    validator.set_context(serializer)
    assert validator.serializer == serializer
    assert validator.serializer.instance == serializer.instance


def test_instance_validator_validation_error():
    validator = InstanceValidator()

    with pytest.raises(ValidationError):
        validator.raise_validation_error({
            'foo': 'bar'
        })


def test_instance_validator_validation_serializer_error():
    validator = InstanceValidator()
    validator.set_context(serializers.Serializer())

    with pytest.raises(serializers.ValidationError):
        validator.raise_validation_error({
            'foo': 'bar'
        })



from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from rdmo.core.exports import XMLResponse
from rdmo.core.permissions import HasModelPermission
from rdmo.core.views import ChoicesViewSet
from rdmo.core.viewsets import CopyModelMixin

from .models import Condition
from .renderers import ConditionRenderer
from .serializers.export import ConditionExportSerializer
from .serializers.v1 import ConditionIndexSerializer, ConditionSerializer


class ConditionViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = Condition.objects.select_related('source', 'target_option') \
                                .prefetch_related('optionsets', 'questionsets', 'questions', 'tasks')
    serializer_class = ConditionSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'uri',
        'key',
        'source',
        'relation',
        'target_text',
        'target_option'
    )

    @action(detail=False)
    def index(self, request):
        queryset = self.get_queryset()
        serializer = ConditionIndexSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = ConditionExportSerializer(self.get_queryset(), many=True)
        xml = ConditionRenderer().render(serializer.data)
        return XMLResponse(xml, name='conditions')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = ConditionExportSerializer(self.get_object())
        xml = ConditionRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().key)


class RelationViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = Condition.RELATION_CHOICES



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-10 16:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0034_remove_attributeentity_conditions'),
        ('questions', '0022_data_migration')
    ]

    operations = [
        migrations.RemoveField(
            model_name='attributeentity',
            name='is_collection',
        ),
        migrations.RemoveField(
            model_name='attributeentity',
            name='parent_collection',
        ),
    ]



import logging

from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic.base import View
from rdmo.core.imports import handle_uploaded_file
from rdmo.core.xml import flat_xml_to_elements, read_xml_file

from .imports import check_permissions, import_elements

logger = logging.getLogger(__name__)


class UploadView(LoginRequiredMixin, View):

    def get_success_url(self):
        return self.request.META.get('HTTP_REFERER') or reverse('management')

    def get(self, request):
        return HttpResponseRedirect(reverse('management'))

    def post(self, request):
        try:
            uploaded_file = request.FILES['uploaded_file']
        except KeyError:
            return HttpResponseRedirect(self.get_success_url())
        else:
            import_tmpfile_name = handle_uploaded_file(uploaded_file)

        root = read_xml_file(import_tmpfile_name)
        if root is None:
            logger.info('Xml parsing error. Import failed.')
            return render(request, 'core/error.html', {
                'title': _('Import error'),
                'errors': [_('The content of the xml file does not consist of well formed data or markup.')]
            }, status=400)

        else:
            try:
                elements = flat_xml_to_elements(root)
            except (KeyError, TypeError):
                return render(request, 'core/error.html', {
                    'title': _('Import error'),
                    'errors': [_('This is not a RDMO XML file.')]
                }, status=400)

            if check_permissions(elements, request.user):
                # store information in session for ProjectCreateImportView
                request.session['import_file_name'] = uploaded_file.name
                request.session['import_tmpfile_name'] = import_tmpfile_name
                request.session['import_success_url'] = self.get_success_url()

                return render(request, 'management/upload.html', {
                    'file_name': uploaded_file.name,
                    'elements': import_elements(elements)
                })
            else:
                return render(request, 'core/error.html', {
                    'title': _('Import error'),
                    'errors': [_('Forbidden.')]
                }, status=403)


class ImportView(LoginRequiredMixin, View):

    def get_success_url(self):
        return self.request.session.get('import_success_url') or reverse('management')

    def get(self, request):
        return HttpResponseRedirect(reverse('management'))

    def post(self, request):
        impor_file_name = request.session['import_file_name']
        import_tmpfile_name = request.session.get('import_tmpfile_name')

        # parse the form data, which is <uri: [parent, checked]> or <uri: [checked]>
        parents = {}
        checked = {}
        for key, values in request.POST.lists():
            if key.startswith('http'):
                try:
                    parents[key] = None if values[0] == 'null' else values[0]
                    checked[key] = True if values[1] == 'on' else False
                except IndexError:
                    parents[key] = False
                    checked[key] = True if values[0] == 'on' else False

        root = read_xml_file(import_tmpfile_name)
        if root is None:
            logger.info('Xml parsing error. Import failed.')
            return render(request, 'core/error.html', {
                'title': _('Import error'),
                'errors': [_('The content of the xml file does not consist of well formed data or markup.')]
            }, status=400)

        else:
            try:
                elements = flat_xml_to_elements(root)
            except (KeyError, TypeError):
                return render(request, 'core/error.html', {
                    'title': _('Import error'),
                    'errors': [_('This is not a RDMO XML file.')]
                }, status=400)

            if check_permissions(elements, request.user):
                if checked:
                    return render(request, 'management/import.html', {
                        'file_name': impor_file_name,
                        'elements': import_elements(elements, parents=parents, save=checked),
                        'success_url': self.get_success_url()
                    })
                else:
                    # if nothing was checked, just return to the success_url
                    return HttpResponseRedirect(self.get_success_url())

            else:
                return render(request, 'core/error.html', {
                    'title': _('Import error'),
                    'errors': [_('Forbidden.')]
                }, status=403)



import os

from rdmo.core.xml import flat_xml_to_elements, read_xml_file
from rdmo.domain.models import Attribute
from rdmo.management.imports import import_elements
from rdmo.options.models import Option


def test_non_unique_path(db, settings):
    count = Attribute.objects.count()

    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'domain-non-unique-path.xml')
    root = read_xml_file(xml_file)
    elements = flat_xml_to_elements(root)
    checked = {element.get('uri'): True for element in elements}
    instances = import_elements(elements, parents={}, save=checked)

    # one instance has an error
    assert len([instance.errors for instance in instances if instance.errors]) == 1

    # two instances have no error
    assert len([instance.errors for instance in instances if not instance.errors]) == 2

    # only 2 attributes have been imported
    assert Attribute.objects.count() == count + 2


def test_non_unique_key(db, settings):
    count = Option.objects.count()

    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'options-non-unique-key.xml')
    root = read_xml_file(xml_file)
    elements = flat_xml_to_elements(root)
    checked = {element.get('uri'): True for element in elements}
    instances = import_elements(elements, parents={}, save=checked)

    # one instance has an error
    assert len([instance.errors for instance in instances if instance.errors]) == 1

    # one instance has no error
    assert len([instance.errors for instance in instances if not instance.errors]) == 1

    # no option has been imported
    assert Option.objects.count() == count


def test_missing_parent(db, settings):
    count = Option.objects.count()

    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'options-missing-parent.xml')
    root = read_xml_file(xml_file)
    elements = flat_xml_to_elements(root)
    checked = {element.get('uri'): True for element in elements}
    instances = import_elements(elements, parents={}, save=checked)

    # one instance has an error
    assert len([instance.errors for instance in instances if instance.errors]) == 1

    # no option has been imported
    assert Option.objects.count() == count



from django import forms
from django.contrib import admin
from django.db import models

from rdmo.core.utils import get_language_fields

from .models import Catalog, Question, QuestionSet, Section
from .validators import (CatalogLockedValidator, CatalogUniqueURIValidator,
                         QuestionLockedValidator, QuestionSetLockedValidator,
                         QuestionSetUniqueURIValidator,
                         QuestionUniqueURIValidator, SectionLockedValidator,
                         SectionUniqueURIValidator, QuestionSetQuestionSetValidator)
from .utils import get_widget_type_choices


class CatalogAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = Catalog
        fields = '__all__'

    def clean(self):
        CatalogUniqueURIValidator(self.instance)(self.cleaned_data)
        CatalogLockedValidator(self.instance)(self.cleaned_data)


class SectionAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = Section
        fields = '__all__'

    def clean(self):
        SectionUniqueURIValidator(self.instance)(self.cleaned_data)
        SectionLockedValidator(self.instance)(self.cleaned_data)


class QuestionSetAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = QuestionSet
        fields = '__all__'

    def clean(self):
        QuestionSetUniqueURIValidator(self.instance)(self.cleaned_data)
        QuestionSetQuestionSetValidator(self.instance)(self.cleaned_data)
        QuestionSetLockedValidator(self.instance)(self.cleaned_data)


class QuestionAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)
    widget_type = forms.ChoiceField(choices=get_widget_type_choices())

    class Meta:
        model = Question
        fields = '__all__'

    def clean(self):
        QuestionUniqueURIValidator(self.instance)(self.cleaned_data)
        QuestionLockedValidator(self.instance)(self.cleaned_data)


class CatalogAdmin(admin.ModelAdmin):
    form = CatalogAdminForm

    search_fields = ['uri'] + get_language_fields('title')
    list_display = ('uri', 'title', 'projects_count', 'available')
    readonly_fields = ('uri', )
    list_filter = ('available', )

    def get_queryset(self, request):
        return super().get_queryset(request) \
                      .annotate(projects_count=models.Count('projects'))

    def projects_count(self, obj):
        return obj.projects_count


class SectionAdmin(admin.ModelAdmin):
    form = SectionAdminForm

    search_fields = ['uri'] + get_language_fields('title')
    list_display = ('uri', 'title')
    readonly_fields = ('uri', 'path')
    list_filter = ('catalog', )


class QuestionSetAdmin(admin.ModelAdmin):
    form = QuestionSetAdminForm

    search_fields = ['uri'] + get_language_fields('title') + get_language_fields('help')
    list_display = ('uri', 'attribute', 'is_collection')
    readonly_fields = ('uri', 'path')
    list_filter = ('section__catalog', 'section', 'is_collection')


class QuestionItemAdmin(admin.ModelAdmin):
    form = QuestionAdminForm

    search_fields = ['uri'] + get_language_fields('help') + get_language_fields('text')
    list_display = ('uri', 'attribute', 'text', 'is_collection')
    readonly_fields = ('uri', 'path')
    list_filter = ('questionset__section__catalog', 'questionset__section', 'is_collection', 'widget_type', 'value_type')


admin.site.register(Catalog, CatalogAdmin)
admin.site.register(Section, SectionAdmin)
admin.site.register(QuestionSet, QuestionSetAdmin)
admin.site.register(Question, QuestionItemAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-03 13:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0011_path'),
    ]

    operations = [
        migrations.AlterField(
            model_name='questionentity',
            name='path',
            field=models.CharField(blank=True, help_text='The path part of the URI of this question/questionset (auto-generated).', max_length=512, null=True, verbose_name='Path'),
        ),
        migrations.AlterField(
            model_name='section',
            name='path',
            field=models.CharField(blank=True, help_text='The path part of the URI of this section (auto-generated).', max_length=512, null=True, verbose_name='Label'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='path',
            field=models.CharField(blank=True, help_text='The path part of the URI of this subsection (auto-generated).', max_length=512, null=True, verbose_name='Label'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0003_hint_renamed_to_help_text'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='detailkey',
            options={'verbose_name': 'DetailKey', 'verbose_name_plural': 'DetailKeys', 'ordering': ('key',)},
        ),
        migrations.AlterModelOptions(
            name='profile',
            options={'verbose_name': 'Profile', 'permissions': (('update_profile', 'Can update own profile'),), 'verbose_name_plural': 'Profiles', 'ordering': ('user',)},
        ),
        migrations.AlterField(
            model_name='detailkey',
            name='help_text',
            field=models.TextField(blank=True, help_text='Enter a help text to be displayed next to the input element'),
        ),
    ]



# import os
# import xml.etree.ElementTree as ET

# from ..imports import read_xml_file
# from ..xml import (flat_xml_to_dictlist, get_ns_map, get_ns_tag,
#                    get_text_or_attribute, get_uri, node_type_from_dictlist,
#                    sort_dictlist_by_key)


# # tests
# def test_get_ns_map():
#     xmls = read_all_test_xmls()
#     for key in xmls:
#         assert str(get_ns_map(xmls[key])) == "{'dc': 'http://purl.org/dc/elements/1.1/'}"


# def test_get_ns_tag():
#     root = read_test_xml('domain')
#     nsmap = get_ns_map(root)
#     assert get_ns_tag('dc:uri', nsmap) == "{http://purl.org/dc/elements/1.1/}uri"


# def test_get_text_or_attribute():
#     root = read_test_xml('domain')
#     el = root.find('attribute')
#     assert get_text_or_attribute(el, 'key') == 'set'
#     assert get_text_or_attribute(el, 'does_not_exist') is None


# def test_get_uri():
#     root = read_test_xml('domain')
#     el = root.find('attribute')
#     nsmap = get_ns_map(root)
#     assert get_uri(el, nsmap) == 'http://example.com/terms/domain/set'


# def test_etree_to_dict():
#     root = read_test_xml('domain')
#     dictlist = flat_xml_to_dictlist(root)
#     assert dictlist[0]['uri'] == 'http://example.com/terms/domain/conditions'
#     assert dictlist[1]['parent'] == 'http://example.com/terms/domain/conditions'

#     root = read_test_xml('conditions')
#     dictlist = flat_xml_to_dictlist(root)
#     assert dictlist[0]['uri'] == 'http://example.com/terms/conditions/options_empty'

#     root = read_test_xml('options')
#     dictlist = flat_xml_to_dictlist(root)
#     assert dictlist[0]['uri'] == 'http://example.com/terms/options/one_two_three'

#     root = read_test_xml('tasks')
#     dictlist = flat_xml_to_dictlist(root)
#     assert dictlist[0]['uri'] == 'http://example.com/terms/tasks/options_contains_one'

#     root = read_test_xml('questions')
#     dictlist = flat_xml_to_dictlist(root)
#     assert dictlist[0]['uri'] == 'http://example.com/terms/questions/catalog'

#     root = read_test_xml('views')
#     dictlist = flat_xml_to_dictlist(root)
#     assert dictlist[0]['uri'] == 'http://example.com/terms/views/view_a'


# def test_node_type_from_dictlist():
#     root = read_test_xml('options')
#     dictlist = flat_xml_to_dictlist(root)
#     optionsets = node_type_from_dictlist(dictlist, 'optionset')
#     assert optionsets[0]['node_type'] == 'optionset'


# # test utils
# def read_all_test_xmls():
#     xml = {}
#     xml['domain'] = read_test_xml('domain')
#     xml['conditions'] = read_test_xml('conditions')
#     xml['options'] = read_test_xml('options')
#     xml['project'] = read_test_xml('project')
#     xml['questions'] = read_test_xml('questions')
#     xml['tasks'] = read_test_xml('tasks')
#     xml['views'] = read_test_xml('views')
#     return xml


# def read_test_xml(filename):
#     xml_folder = os.getcwd().split('rdmo')[0] + 'rdmo/testing/xml/'
#     return read_xml_file(xml_folder + filename + '.xml')



from django.utils.timezone import now, get_current_timezone
from io import StringIO

from django.utils.xmlutils import SimplerXMLGenerator
from django.utils.encoding import smart_str
from rest_framework.renderers import BaseRenderer


class BaseXMLRenderer(BaseRenderer):

    media_type = 'application/xml'
    format = 'xml'

    def render(self, data):

        if data is None:
            return ''

        stream = StringIO()

        xml = SimplerXMLGenerator(stream, "utf-8")
        xml.startDocument()
        self.render_document(xml, data)
        xml.endDocument()
        return stream.getvalue()

    def render_text_element(self, xml, tag, attrs, text):
        # remove None values from attrs
        attrs = dict((key, value) for key, value in attrs.items() if value)

        xml.startElement(tag, attrs)
        if text is not None:
            xml.characters(smart_str(text))
        xml.endElement(tag)

    def render_document(self, xml, data):
        pass

    @property
    def created(self):
        return now().astimezone(get_current_timezone()).isoformat()



import logging

from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.views.generic import CreateView, DetailView, UpdateView

from rdmo.core.views import ObjectPermissionMixin, RedirectViewMixin

from ..forms import SnapshotCreateForm
from ..models import Project, Snapshot

logger = logging.getLogger(__name__)


class SnapshotCreateView(ObjectPermissionMixin, RedirectViewMixin, CreateView):
    model = Snapshot
    form_class = SnapshotCreateForm
    permission_required = 'projects.add_snapshot_object'

    def dispatch(self, *args, **kwargs):
        self.project = get_object_or_404(Project.objects.all(), pk=self.kwargs['project_id'])
        return super(SnapshotCreateView, self).dispatch(*args, **kwargs)

    def get_permission_object(self):
        return self.project

    def get_form_kwargs(self):
        kwargs = super(SnapshotCreateView, self).get_form_kwargs()
        kwargs['project'] = self.project
        return kwargs


class SnapshotUpdateView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    model = Snapshot
    queryset = Snapshot.objects.all()
    fields = ['title', 'description']
    permission_required = 'projects.change_snapshot_object'

    def get_queryset(self):
        return Snapshot.objects.filter(project_id=self.kwargs['project_id'])

    def get_permission_object(self):
        return self.get_object().project


class SnapshotRollbackView(ObjectPermissionMixin, RedirectViewMixin, DetailView):
    model = Snapshot
    queryset = Snapshot.objects.all()
    permission_required = 'projects.rollback_snapshot_object'
    template_name = 'projects/snapshot_rollback.html'

    def get_queryset(self):
        return Snapshot.objects.filter(project_id=self.kwargs['project_id'])

    def get_permission_object(self):
        return self.get_object().project

    def post(self, request, *args, **kwargs):
        snapshot = self.get_object()

        if 'cancel' not in request.POST:
            snapshot.rollback()

        return HttpResponseRedirect(reverse('project', args=[snapshot.project.id]))



import mimetypes
from pathlib import Path

import iso8601
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_cleanup import cleanup

from rdmo.core.constants import (VALUE_TYPE_BOOLEAN, VALUE_TYPE_CHOICES,
                                 VALUE_TYPE_DATETIME, VALUE_TYPE_TEXT)
from rdmo.core.models import Model
from rdmo.domain.models import Attribute
from rdmo.options.models import Option

from ..managers import ValueManager
from ..utils import get_value_path


def get_file_upload_to(instance, filename):
    return str(get_value_path(instance.project, instance.snapshot) / str(instance.id) / filename)


class Value(Model):

    objects = ValueManager()

    FALSE_TEXT = [None, '', '0', 'f', 'F', 'false', 'False']

    project = models.ForeignKey(
        'Project', on_delete=models.CASCADE, related_name='values',
        verbose_name=_('Project'),
        help_text=_('The project this value belongs to.')
    )
    snapshot = models.ForeignKey(
        'Snapshot', blank=True, null=True,
        on_delete=models.CASCADE, related_name='values',
        verbose_name=_('Snapshot'),
        help_text=_('The snapshot this value belongs to.')
    )
    attribute = models.ForeignKey(
        Attribute, blank=True, null=True,
        on_delete=models.SET_NULL, related_name='values',
        verbose_name=_('Attribute'),
        help_text=_('The attribute this value belongs to.')
    )
    set_prefix = models.CharField(
        max_length=16, blank=True, default='',
        verbose_name=_('Set prefix'),
        help_text=_('The position of this value with respect to superior sets (i.e. for nested question sets).')
    )
    set_index = models.IntegerField(
        default=0,
        verbose_name=_('Set index'),
        help_text=_('The position of this value in a set (i.e. for a question set tagged as collection).')
    )
    collection_index = models.IntegerField(
        default=0,
        verbose_name=_('Collection index'),
        help_text=_('The position of this value in a list (i.e. for a question tagged as collection).')
    )
    text = models.TextField(
        blank=True,
        verbose_name=_('Text'),
        help_text=_('The string stored for this value.')
    )
    option = models.ForeignKey(
        Option, blank=True, null=True, on_delete=models.SET_NULL, related_name='values',
        verbose_name=_('Option'),
        help_text=_('The option stored for this value.')
    )
    file = models.FileField(
        upload_to=get_file_upload_to, null=True, blank=True,
        verbose_name=_('File'),
        help_text=_('The file stored for this value.')
    )
    value_type = models.CharField(
        max_length=8, choices=VALUE_TYPE_CHOICES, default=VALUE_TYPE_TEXT,
        verbose_name=_('Value type'),
        help_text=_('Type of this value.')
    )
    unit = models.CharField(
        max_length=64, blank=True,
        verbose_name=_('Unit'),
        help_text=_('Unit for this value.')
    )
    external_id = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('External id'),
        help_text=_('External id for this value.')
    )

    class Meta:
        ordering = ('attribute', 'set_index', 'collection_index')
        verbose_name = _('Value')
        verbose_name_plural = _('Values')

    def __str__(self):
        return '{} / {} / {} / {} / {}'.format(
            self.project, self.snapshot or '-', self.set_prefix, self.set_index, self.collection_index
        )

    @property
    def as_dict(self):
        value_dict = {
            'id': self.id,
            'created': self.created,
            'updated': self.updated,
            'set_prefix': self.set_prefix,
            'set_index': self.set_index,
            'collection_index': self.collection_index,
            'value_type': self.value_type,
            'unit': self.unit,
            'external_id': self.external_id,
            'value': self.value,
            'value_and_unit': self.value_and_unit,
            'is_true': self.is_true,
            'is_false': self.is_false,
            'is_empty': self.is_empty,
            'as_number': self.as_number
        }

        if self.file:
            value_dict.update({
                'file_name': self.file_name,
                'file_url': self.file_url,
                'file_type': self.file_type,
                'file_path': self.file_path
            })

        return value_dict

    @property
    def value(self):
        if self.option:
            value = self.option.text or ''
            if self.option.additional_input and self.text:
                value += ': ' + self.text
            return value

        elif self.file:
            return self.file_name

        elif self.text:
            if self.value_type == VALUE_TYPE_DATETIME:
                try:
                    return iso8601.parse_date(self.text).date()
                except iso8601.ParseError:
                    return self.text
            elif self.value_type == VALUE_TYPE_BOOLEAN:
                if self.text == '1':
                    return _('Yes')
                else:
                    return _('No')
            else:
                return self.text
        else:
            return None

    @property
    def value_and_unit(self):
        value = self.value

        if value is None:
            return ''
        elif self.unit:
            return '%s %s' % (value, self.unit)
        else:
            return value

    @property
    def is_true(self):
        return any([
            self.text not in self.FALSE_TEXT,
            self.option,
            self.file,
            self.external_id != ''
        ])

    @property
    def is_false(self):
        return all([
            self.text in self.FALSE_TEXT,
            not self.option,
            not self.file,
            self.external_id == ''
        ])

    @property
    def is_empty(self):
        return all([
            self.text == '',
            not self.option,
            not self.file,
            self.external_id == ''
        ])

    @property
    def as_number(self):
        try:
            val = self.text
        except AttributeError:
            return 0
        else:
            if isinstance(val, str):
                val = val.replace(',', '.')

            if isinstance(val, float) is False:
                try:
                    return int(val)
                except (ValueError, TypeError):
                    pass
                try:
                    return float(val)
                except (ValueError, TypeError):
                    return 0
            else:
                return val

    @property
    def file_name(self):
        if self.file:
            return Path(self.file.name).name

    @property
    def file_url(self):
        if self.file:
            return reverse('v1-projects:value-file', args=[self.id])

    @property
    def file_type(self):
        if self.file:
            return mimetypes.guess_type(self.file.name)[0]

    @property
    def file_path(self):
        if self.file:
            resource_path = get_value_path(self.project, self.snapshot)
            return Path(self.file.name).relative_to(resource_path).as_posix()

    def copy_file(self, file_name, file_content):
        # copies a file field from a different value over to this value
        # this is tricky, because we need to trick django_cleanup to not delete the original file
        # important for snapshots and import from projects
        self.file.save(file_name, file_content, save=False)
        cleanup.refresh(self)
        self.save()



from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from rdmo.core.models import Model

from ..managers import SnapshotManager


class Snapshot(Model):

    objects = SnapshotManager()

    project = models.ForeignKey(
        'Project', related_name='snapshots',
        on_delete=models.CASCADE, null=True,
        verbose_name=_('Project'),
        help_text=_('The project this snapshot belongs to.')
    )
    title = models.CharField(
        max_length=256,
        verbose_name=_('Title'),
        help_text=_('The title for this snapshot.')
    )
    description = models.TextField(
        blank=True,
        verbose_name=_('Description'),
        help_text=_('A description for this snapshot (optional).')
    )

    class Meta:
        ordering = ('project', '-created')
        verbose_name = _('Snapshot')
        verbose_name_plural = _('Snapshots')

    def __str__(self):
        return '%s / %s' % (self.project.title, self.title)

    def get_absolute_url(self):
        return reverse('project', kwargs={'pk': self.project.pk})

    def save(self, *args, **kwargs):
        # copy_values is set to True for creating new snapshots and to False for updating
        # for imports it is provided as kwarg (as False)
        copy_values = kwargs.pop('copy_values', self.pk is None)
        super().save()

        if copy_values:
            # loop over values without snapshot and save a copy with a fk to the snapshot
            for value in self.project.values.filter(snapshot=None):
                value.pk = None
                value.snapshot = self
                value.save()

                if value.file:
                    value.copy_file(value.file_name, value.file)

    def rollback(self):
        # remove all current values for this project
        self.project.values.filter(snapshot=None).delete()

        # remove the snapshot_id from this snapshots values so they are current values
        for value in self.values.all():
            value.snapshot = None
            value.save()

            if value.file:
                # here, it is ok that django_cleanup deletes the file
                value.file.save(value.file_name, value.file)

        # remove all snapshot created later and the current_snapshot
        # this also removes the values of these snapshots
        for snapshot in self.project.snapshots.filter(created__gte=self.created):
            snapshot.delete()



import logging

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView, TemplateView

from rdmo.core.exports import XMLResponse
from rdmo.core.utils import get_model_field_meta, render_to_format
from rdmo.core.views import CSRFViewMixin, ModelPermissionMixin

from .models import Option, OptionSet
from .renderers import OptionSetRenderer
from .serializers.export import OptionSetExportSerializer

log = logging.getLogger(__name__)


class OptionsView(ModelPermissionMixin, CSRFViewMixin, TemplateView):
    template_name = 'options/options.html'
    permission_required = 'options.view_option'

    def get_context_data(self, **kwargs):
        context = super(OptionsView, self).get_context_data(**kwargs)
        context['export_formats'] = settings.EXPORT_FORMATS
        context['meta'] = {
            'OptionSet': get_model_field_meta(OptionSet),
            'Option': get_model_field_meta(Option)
        }
        return context


class OptionsExportView(ModelPermissionMixin, ListView):
    model = OptionSet
    context_object_name = 'optionsets'
    permission_required = 'options.view_optionset'

    def render_to_response(self, context, **response_kwargs):
        format = self.kwargs.get('format')
        if format == 'xml':
            serializer = OptionSetExportSerializer(context['optionsets'], many=True)
            xml = OptionSetRenderer().render(serializer.data)
            return XMLResponse(xml, name='options')
        else:
            return render_to_format(self.request, format, _('Options'), 'options/options_export.html', context)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-10 12:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0020_data_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='questionentity',
            name='is_collection',
            field=models.BooleanField(default=False, help_text='Designates whether this question/questionset is a collection.', verbose_name='is collection'),
        ),
    ]



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import QuestionSet, Section
from ..serializers.v1 import QuestionSetSerializer
from ..validators import QuestionSetUniqueURIValidator


def test_unique_uri_validator_create(db):
    QuestionSetUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'section': Section.objects.first()
    })


def test_unique_uri_validator_create2(db):
    section = Section.objects.first()
    questionset = section.questionsets.first()

    QuestionSetUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'section': section,
        'questionset': questionset,
    })


def test_unique_uri_validator_create_error(db):
    section = Section.objects.first()

    with pytest.raises(ValidationError):
        QuestionSetUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': section.questionsets.first().key,
            'section': section
        })


def test_unique_uri_validator_create_error2(db):
    section = Section.objects.first()
    questionset = section.questionsets.first()

    with pytest.raises(ValidationError):
        QuestionSetUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questions.first().key,
            'section': section,
            'questionset': questionset,
        })


def test_unique_uri_validator_create_error3(db):
    # get a questionset which contains a questionset
    questionset = QuestionSet.objects.exclude(questionset=None).first().questionset

    with pytest.raises(ValidationError):
        QuestionSetUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questions.first().key,
            'section': questionset.section,
            'questionset': questionset,
        })


def test_unique_uri_validator_update(db):
    questionset = QuestionSet.objects.first()

    QuestionSetUniqueURIValidator(questionset)({
        'uri_prefix': questionset.uri_prefix,
        'key': questionset.key,
        'section': questionset.section
    })


def test_unique_uri_validator_update_error(db):
    questionset = QuestionSet.objects.first()

    with pytest.raises(ValidationError):
        QuestionSetUniqueURIValidator(questionset)({
            'uri_prefix': questionset.uri_prefix,
            'key': questionset.section.questionsets.last().key,
            'section': questionset.section
        })


def test_unique_uri_validator_serializer_create(db):
    validator = QuestionSetUniqueURIValidator()
    validator.set_context(QuestionSetSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'section': Section.objects.first()
    })


def test_unique_uri_validator_serializer_create2(db):
    section = Section.objects.first()
    questionset = section.questionsets.first()

    validator = QuestionSetUniqueURIValidator()
    validator.set_context(QuestionSetSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'section': section,
        'questionset': questionset
    })


def test_unique_uri_validator_serializer_create_error(db):
    section = Section.objects.first()

    validator = QuestionSetUniqueURIValidator()
    validator.set_context(QuestionSetSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': section.questionsets.last().key,
            'section': section
        })


def test_unique_uri_validator_serializer_create_error2(db):
    section = Section.objects.first()
    questionset = section.questionsets.first()

    validator = QuestionSetUniqueURIValidator()
    validator.set_context(QuestionSetSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questions.first().key,
            'section': section,
            'questionset': questionset,
        })


def test_unique_uri_validator_serializer_create_error3(db):
    # get a questionset which contains a questionset
    questionset = QuestionSet.objects.exclude(questionset=None).first().questionset

    validator = QuestionSetUniqueURIValidator()
    validator.set_context(QuestionSetSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questions.first().key,
            'section': questionset.section,
            'questionset': questionset,
        })


def test_unique_uri_validator_serializer_update(db):
    questionset = QuestionSet.objects.first()

    validator = QuestionSetUniqueURIValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    validator({
        'uri_prefix': questionset.uri_prefix,
        'key': questionset.key,
        'section': questionset.section
    })


def test_unique_uri_validator_serializer_update_error(db):
    questionset = QuestionSet.objects.first()

    validator = QuestionSetUniqueURIValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': questionset.uri_prefix,
            'key': questionset.section.questionsets.last().key,
            'section': questionset.section
        })



from django.conf import settings
from django.contrib.auth import get_user_model
from rest_framework import serializers

from rdmo.services.validators import ProviderValidator

from ...models import (Integration, IntegrationOption, Issue, IssueResource,
                       Membership, Project, Snapshot, Value)
from ...validators import ValueValidator


class UserSerializer(serializers.ModelSerializer):

    class Meta:
        model = get_user_model()
        fields = [
            'id',
        ]
        if settings.USER_API:
            fields += [
                'username',
                'first_name',
                'last_name',
                'email'
            ]


class ProjectSerializer(serializers.ModelSerializer):

    class ParentField(serializers.PrimaryKeyRelatedField):

        def get_queryset(self):
            return Project.objects.filter_user(self.context['request'].user)

    parent = ParentField(required=False)

    owners = UserSerializer(many=True, read_only=True)
    managers = UserSerializer(many=True, read_only=True)
    authors = UserSerializer(many=True, read_only=True)
    guests = UserSerializer(many=True, read_only=True)

    class Meta:
        model = Project
        fields = (
            'id',
            'title',
            'description',
            'catalog',
            'snapshots',
            'parent',
            'owners',
            'managers',
            'authors',
            'guests'
        )
        read_only_fields = (
            'snapshots',
        )


class ProjectMembershipSerializer(serializers.ModelSerializer):

    class Meta:
        model = Membership
        fields = (
            'id',
            'user',
            'role'
        )


class ProjectMembershipUpdateSerializer(serializers.ModelSerializer):

    class Meta:
        model = Membership
        fields = (
            'role',
        )


class ProjectIntegrationOptionSerializer(serializers.ModelSerializer):

    class Meta:
        model = IntegrationOption
        fields = (
            'key',
            'value'
        )


class ProjectIntegrationSerializer(serializers.ModelSerializer):

    options = ProjectIntegrationOptionSerializer(many=True)

    class Meta:
        model = Integration
        fields = (
            'id',
            'provider_key',
            'options'
        )
        validators = [
            ProviderValidator()
        ]

    def create(self, validated_data):
        provider_key = validated_data.get('provider_key')
        project = validated_data.get('project')
        options = {option.get('key'): option.get('value') for option in validated_data.get('options', [])}

        integration = Integration(project=project, provider_key=provider_key)
        integration.save()
        integration.save_options(options)

        return integration

    def update(self, integration, validated_data):
        options = {option.get('key'): option.get('value') for option in validated_data.get('options', [])}

        integration.save_options(options)

        return integration


class ProjectIssueResourceSerializer(serializers.ModelSerializer):

    integration = serializers.PrimaryKeyRelatedField(read_only=True)

    class Meta:
        model = IssueResource
        fields = (
            'id',
            'integration',
            'url'
        )


class ProjectIssueSerializer(serializers.ModelSerializer):

    task = serializers.PrimaryKeyRelatedField(read_only=True)
    resources = ProjectIssueResourceSerializer(read_only=True, many=True)

    class Meta:
        model = Issue
        fields = (
            'id',
            'task',
            'status',
            'resources'
        )


class ProjectSnapshotSerializer(serializers.ModelSerializer):

    class Meta:
        model = Snapshot
        fields = (
            'id',
            'title',
            'description'
        )


class ProjectValueSerializer(serializers.ModelSerializer):

    class Meta:
        model = Value
        fields = (
            'id',
            'created',
            'updated',
            'attribute',
            'set_prefix',
            'set_index',
            'collection_index',
            'text',
            'option',
            'file_name',
            'file_url',
            'value_type',
            'unit',
            'external_id'
        )
        validators = (ValueValidator(), )


class MembershipSerializer(serializers.ModelSerializer):

    class Meta:
        model = Membership
        fields = (
            'id',
            'project',
            'user',
            'role'
        )


class IntegrationSerializer(serializers.ModelSerializer):

    options = ProjectIntegrationOptionSerializer(many=True, read_only=True)

    class Meta:
        model = Integration
        fields = (
            'id',
            'project',
            'provider_key',
            'options'
        )


class IssueResourceSerializer(serializers.ModelSerializer):

    integration = serializers.PrimaryKeyRelatedField(read_only=True)

    class Meta:
        model = IssueResource
        fields = (
            'id',
            'integration',
            'url'
        )


class IssueSerializer(serializers.ModelSerializer):

    project = serializers.PrimaryKeyRelatedField(read_only=True)
    task = serializers.PrimaryKeyRelatedField(read_only=True)
    resources = IssueResourceSerializer(read_only=True, many=True)

    class Meta:
        model = Issue
        fields = (
            'id',
            'project',
            'task',
            'status',
            'resources'
        )


class SnapshotSerializer(serializers.ModelSerializer):

    class Meta:
        model = Snapshot
        fields = (
            'id',
            'project',
            'title',
            'description'
        )


class ValueSerializer(serializers.ModelSerializer):

    class Meta:
        model = Value
        fields = (
            'id',
            'created',
            'updated',
            'project',
            'snapshot',
            'attribute',
            'set_prefix',
            'set_index',
            'collection_index',
            'text',
            'option',
            'file_name',
            'file_url',
            'value_type',
            'unit',
            'external_id'
        )



from django.urls import reverse


def test_task_search(db, client):
    client.login(username='admin', password='admin')

    url = reverse('admin:tasks_task_changelist') + '?q=test'
    response = client.get(url)
    assert response.status_code == 200



import pytest
from django.urls import reverse

from ..models import Overlay

users = (
    ('user', 'user'),
    ('anonymous', None)
)


@pytest.mark.parametrize('username,password', users)
def test_reset_overlays_get(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('reset_overlays')
    response = client.get(url)
    if password:
        assert response.status_code == 200
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
def test_reset_overlays_post(db, client, username, password):
    client.login(username=username, password=password)

    if password:
        assert Overlay.objects.filter(user__username=username).exists() is True

    url = reverse('reset_overlays')
    response = client.post(url)
    if password:
        assert response.status_code == 302
        assert Overlay.objects.filter(user__username=username).exists() is False
    else:
        assert response.status_code == 302



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-03 11:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0013_mptt'),
    ]

    operations = [
        migrations.AddField(
            model_name='attributeentity',
            name='is_attribute',
            field=models.BooleanField(default=False),
        ),
    ]



from django.conf import settings
from django.template import RequestContext, Template
from django.urls import reverse
from django.utils import translation


def test_i18n_switcher(rf):
    """ The language switcher is rendered correctly. """

    # create a fake template with a name
    template = "{% load core_tags %}{% i18n_switcher %}"

    # set a language
    translation.activate(settings.LANGUAGES[0][0])

    # render the link
    request = rf.get(reverse('home'))
    context = RequestContext(request, {})
    rendered_template = Template(template).render(context)
    for language in settings.LANGUAGES:
        if language == settings.LANGUAGES[0]:
            assert '<a href="/i18n/%s/"><u>%s</u></a>' % language in rendered_template
        else:
            assert'<a href="/i18n/%s/">%s</a>' % language in rendered_template



import logging

from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.sites.shortcuts import get_current_site
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic import CreateView, TemplateView
from rdmo.core.views import RedirectViewMixin
from rdmo.questions.models import Catalog
from rdmo.tasks.models import Task
from rdmo.views.models import View

from ..forms import ProjectForm
from ..mixins import ProjectImportMixin
from ..models import Membership, Project

logger = logging.getLogger(__name__)


class ProjectCreateView(LoginRequiredMixin, RedirectViewMixin, CreateView):
    model = Project
    form_class = ProjectForm

    def get_form_kwargs(self):
        catalogs = Catalog.objects.filter_current_site() \
                                  .filter_group(self.request.user) \
                                  .filter_availability(self.request.user)
        projects = Project.objects.filter_user(self.request.user)

        form_kwargs = super().get_form_kwargs()
        form_kwargs.update({
            'catalogs': catalogs,
            'projects': projects
        })
        return form_kwargs

    def form_valid(self, form):
        # add current site
        form.instance.site = get_current_site(self.request)

        # save the project
        response = super(ProjectCreateView, self).form_valid(form)

        # add all tasks to project
        tasks = Task.objects.filter_current_site() \
                            .filter_group(self.request.user) \
                            .filter_availability(self.request.user)
        for task in tasks:
            form.instance.tasks.add(task)

        # add all views to project
        views = View.objects.filter_current_site() \
                            .filter_catalog(self.object.catalog) \
                            .filter_group(self.request.user) \
                            .filter_availability(self.request.user)
        for view in views:
            form.instance.views.add(view)

        # add current user as owner
        membership = Membership(project=form.instance, user=self.request.user, role='owner')
        membership.save()

        return response


class ProjectCreateImportView(ProjectImportMixin, LoginRequiredMixin, TemplateView):
    success_url = reverse_lazy('projects')

    def get(self, request, *args, **kwargs):
        return HttpResponseRedirect(self.success_url)

    def post(self, request, *args, **kwargs):
        self.object = None

        method = request.POST.get('method')
        if method in ['upload_file', 'import_file']:
            response = getattr(self, method)()
        else:
            response = None

        if response is None:
            return render(request, 'core/error.html', {
                'title': _('Import error'),
                'errors': [_('There has been an error with your import.')]
            }, status=400)
        else:
            return response



from django.conf import settings
from django.db import models

from .constants import PERMISSIONS


class CurrentSiteQuerySetMixin(object):

    def filter_current_site(self):
        return self.filter(models.Q(sites=None) | models.Q(sites=settings.SITE_ID))


class GroupsQuerySetMixin(object):

    def filter_group(self, user):
        groups = user.groups.all()
        return self.filter(models.Q(groups=None) | models.Q(groups__in=groups))


class AvailabilityQuerySetMixin(object):

    def filter_availability(self, user):
        model_name = self.model._meta.model_name
        permissions = PERMISSIONS[model_name]

        if user.has_perms(permissions):
            return self
        else:
            return self.filter(available=True)


class CurrentSiteManagerMixin(object):

    def filter_current_site(self):
        return self.get_queryset().filter_current_site()


class GroupsManagerMixin(object):

    def filter_group(self, user):
        return self.get_queryset().filter_group(user)


class AvailabilityManagerMixin(object):

    def filter_availability(self, user):
        return self.get_queryset().filter_availability(user)



from django.urls import re_path

from ..views import DomainExportView, DomainView

urlpatterns = [
    re_path(r'^$', DomainView.as_view(), name='domain'),
    re_path(r'^export/(?P<format>[a-z]+)/$', DomainExportView.as_view(), name='domain_export')
]



# Generated by Django 2.2.13 on 2020-08-20 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0051_sites_blank'),
    ]

    operations = [
        migrations.AddField(
            model_name='catalog',
            name='available',
            field=models.BooleanField(default=True, help_text='Designates whether this catalog is generally available for projects.', verbose_name='Available'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-23 14:24
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0007_ordering'),
    ]

    operations = [
        migrations.AlterField(
            model_name='condition',
            name='title',
            field=models.CharField(max_length=256, validators=[django.core.validators.RegexValidator('^[a-zA-z0-9_]*$', 'Only letters, numbers, or underscores are allowed.')]),
        ),
    ]



# Generated by Django 2.2.18 on 2021-05-20 15:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0060_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='width',
            field=models.IntegerField(blank=True, help_text='Width for the widget of this question (optional, full with: 12).', null=True, verbose_name='Width'),
        ),
    ]



# Generated by Django 2.2.13 on 2020-08-24 11:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0023_require_uri_prefix'),
    ]

    operations = [
        migrations.AlterField(
            model_name='optionset',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='The list of conditions evaluated for this option set.', related_name='optionsets', to='conditions.Condition', verbose_name='Conditions'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-27 15:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0003_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='task',
            options={'ordering': ('key',), 'verbose_name': 'Task', 'verbose_name_plural': 'Tasks'},
        ),
        migrations.AddField(
            model_name='task',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this task.', null=True, verbose_name='Comment'),
        ),
        migrations.AddField(
            model_name='task',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this task. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AddField(
            model_name='task',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this task (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AddField(
            model_name='task',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this task.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
        migrations.AlterField(
            model_name='task',
            name='attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute this task is refering to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='Attribute'),
        ),
        migrations.AlterField(
            model_name='task',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='The list of conditions evaluated for this task.', to='conditions.Condition', verbose_name='Conditions'),
        ),
        migrations.AlterField(
            model_name='task',
            name='text_de',
            field=models.CharField(help_text='The German text for this task.', max_length=256, verbose_name='Text (de)'),
        ),
        migrations.AlterField(
            model_name='task',
            name='text_en',
            field=models.CharField(help_text='The English text for this task.', max_length=256, verbose_name='Text (en)'),
        ),
        migrations.AlterField(
            model_name='task',
            name='time_period',
            field=models.DurationField(help_text='The the time period after this task becomes active.', verbose_name='Time period'),
        ),
        migrations.AlterField(
            model_name='task',
            name='title_de',
            field=models.CharField(help_text='The German title for this task.', max_length=256, verbose_name='Title (de)'),
        ),
        migrations.AlterField(
            model_name='task',
            name='title_en',
            field=models.CharField(help_text='The English title for this task.', max_length=256, verbose_name='Title (en)'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-01-29 16:22
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0037_rename_en_to_lang1'),
    ]

    operations = [
        migrations.RenameField(
            model_name='catalog',
            old_name='title_de',
            new_name='title_lang2',
        ),
        migrations.RenameField(
            model_name='question',
            old_name='help_de',
            new_name='help_lang2',
        ),
        migrations.RenameField(
            model_name='question',
            old_name='text_de',
            new_name='text_lang2',
        ),
        migrations.RenameField(
            model_name='question',
            old_name='verbose_name_de',
            new_name='verbose_name_lang2',
        ),
        migrations.RenameField(
            model_name='question',
            old_name='verbose_name_plural_de',
            new_name='verbose_name_plural_lang2',
        ),
        migrations.RenameField(
            model_name='questionset',
            old_name='help_de',
            new_name='help_lang2',
        ),
        migrations.RenameField(
            model_name='questionset',
            old_name='title_de',
            new_name='title_lang2',
        ),
        migrations.RenameField(
            model_name='questionset',
            old_name='verbose_name_de',
            new_name='verbose_name_lang2',
        ),
        migrations.RenameField(
            model_name='questionset',
            old_name='verbose_name_plural_de',
            new_name='verbose_name_plural_lang2',
        ),
        migrations.RenameField(
            model_name='section',
            old_name='title_de',
            new_name='title_lang2',
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-11 12:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0028_meta'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='project',
            managers=[
            ],
        ),
    ]



from django.conf import settings
from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from rdmo.conditions.models import Condition
from rdmo.core.models import Model, TranslationMixin
from rdmo.core.utils import copy_model, join_url
from rdmo.domain.models import Attribute

from ..managers import QuestionSetManager
from .section import Section


class QuestionSet(Model, TranslationMixin):

    objects = QuestionSetManager()

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this question set (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this question set.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this question set.')
    )
    path = models.CharField(
        max_length=512, blank=True,
        verbose_name=_('Path'),
        help_text=_('The path part of the URI of this question set (auto-generated).')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional internal information about this question set.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this question set (and it\'s questions) can be changed.')
    )
    attribute = models.ForeignKey(
        Attribute, blank=True, null=True, related_name='questionsets',
        on_delete=models.SET_NULL,
        verbose_name=_('Attribute'),
        help_text=_('The attribute this question set belongs to.')
    )
    section = models.ForeignKey(
        Section, on_delete=models.CASCADE, related_name='questionsets',
        verbose_name=_('Section'),
        help_text=_('The section this question set belongs to.')
    )
    questionset = models.ForeignKey(
        'QuestionSet', blank=True, null=True, default=None, on_delete=models.CASCADE, related_name='questionsets',
        verbose_name=_('Question set'),
        help_text=_('The question set this question set belongs to.')
    )
    is_collection = models.BooleanField(
        default=False,
        verbose_name=_('is collection'),
        help_text=_('Designates whether this question set is a collection.')
    )
    order = models.IntegerField(
        default=0,
        verbose_name=_('Order'),
        help_text=_('The position of this question set in lists.')
    )
    title_lang1 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (primary)'),
        help_text=_('The title for this question set in the primary language.')
    )
    title_lang2 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (secondary)'),
        help_text=_('The title for this question set in the secondary language.')
    )
    title_lang3 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (tertiary)'),
        help_text=_('The title for this question set in the tertiary language.')
    )
    title_lang4 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quaternary)'),
        help_text=_('The title for this question set in the quaternary language.')
    )
    title_lang5 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quinary)'),
        help_text=_('The title for this question set in the quinary language.')
    )
    help_lang1 = models.TextField(
        blank=True,
        verbose_name=_('Help (primary)'),
        help_text=_('The help text for this question set in the primary language.')
    )
    help_lang2 = models.TextField(
        blank=True,
        verbose_name=_('Help (secondary)'),
        help_text=_('The help text for this question set in the secondary language.')
    )
    help_lang3 = models.TextField(
        blank=True,
        verbose_name=_('Help (tertiary)'),
        help_text=_('The help text for this question set in the tertiary language.')
    )
    help_lang4 = models.TextField(
        blank=True,
        verbose_name=_('Help (quaternary)'),
        help_text=_('The help text for this question set in the quaternary language.')
    )
    help_lang5 = models.TextField(
        blank=True,
        verbose_name=_('Help (quinary)'),
        help_text=_('The help text for this question set in the quinary language.')
    )
    verbose_name_lang1 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Name (primary)'),
        help_text=_('The name displayed for this question set in the primary language.')
    )
    verbose_name_lang2 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Name (secondary)'),
        help_text=_('The name displayed for this question set in the secondary language.')
    )
    verbose_name_lang3 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Name (tertiary)'),
        help_text=_('The name displayed for this question set in the tertiary language.')
    )
    verbose_name_lang4 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Name (quaternary)'),
        help_text=_('The name displayed for this question set in the quaternary language.')
    )
    verbose_name_lang5 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Name (quinary)'),
        help_text=_('The name displayed for this question set in the quinary language.')
    )
    verbose_name_plural_lang1 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Plural name (primary)'),
        help_text=_('The plural name displayed for this question set in the primary language.')
    )
    verbose_name_plural_lang2 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Plural name (secondary)'),
        help_text=_('The plural name displayed for this question set in the secondary language.')
    )
    verbose_name_plural_lang3 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Plural name (tertiary)'),
        help_text=_('The plural name displayed for this question set in the tertiary language.')
    )
    verbose_name_plural_lang4 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Plural name (quaternary)'),
        help_text=_('The plural name displayed for this question set in the quaternary language.')
    )
    verbose_name_plural_lang5 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Plural name (quinary)'),
        help_text=_('The plural name displayed for this question set in the quinary language.')
    )
    conditions = models.ManyToManyField(
        Condition, blank=True, related_name='questionsets',
        verbose_name=_('Conditions'),
        help_text=_('List of conditions evaluated for this question set.')
    )

    class Meta:
        ordering = ('section', 'order')
        verbose_name = _('Question set')
        verbose_name_plural = _('Question set')

    def __str__(self):
        return self.path

    def save(self, *args, **kwargs):
        self.path = self.build_path(self.key, self.section, self.questionset)
        self.uri = self.build_uri(self.uri_prefix, self.path)

        super().save(*args, **kwargs)

        for questionset in self.questionsets.all():
            questionset.save()
        for question in self.questions.all():
            question.save()

    def copy(self, uri_prefix, key, section=None, questionset=False):
        questionset = copy_model(self, uri_prefix=uri_prefix, key=key,
                                 section=section or self.section,
                                 questionset=self.questionset if questionset is False else questionset,
                                 attribute=self.attribute)

        # copy m2m fields
        questionset.conditions.set(self.conditions.all())

        # copy children
        for child_questionset in self.questionsets.all():
            child_questionset.copy(uri_prefix, child_questionset.key, section=section, questionset=questionset)
        for child_question in self.questions.all():
            child_question.copy(uri_prefix, child_question.key, questionset=questionset)

        return questionset

    @property
    def parent_fields(self):
        return ('section', 'questionset')

    @property
    def title(self):
        return self.trans('title')

    @property
    def help(self):
        return self.trans('help')

    @property
    def verbose_name(self):
        return self.trans('verbose_name')

    @property
    def verbose_name_plural(self):
        return self.trans('verbose_name_plural')

    @property
    def is_locked(self):
        return self.locked or self.section.is_locked

    @property
    def is_question(self):
        return False

    @property
    def has_conditions(self):
        return self.conditions.exists()

    @cached_property
    def elements(self):
        elements = list(self.questionsets.all()) + list(self.questions.all())
        return sorted(elements, key=lambda e: e.order)

    @cached_property
    def neighbors(self):
        ids = list(self.__class__.objects.order_by_catalog(self.section.catalog).values_list('id', flat=True))

        try:
            index = ids.index(self.id)

            if index == 0:
                return {
                    'prev': None,
                    'next': ids[index + 1]
                }
            elif index == len(ids) - 1:
                return {
                    'prev': ids[index - 1],
                    'next': None
                }
            else:
                return {
                    'prev': ids[index - 1],
                    'next': ids[index + 1]
                }

        except (ValueError, IndexError):
            return {
                'prev': None,
                'next': None
            }

    @cached_property
    def next(self):
        return self.neighbors['next']

    @cached_property
    def prev(self):
        return self.neighbors['prev']

    def get_descendants(self, include_self=False):
        # this function tries to mimic the same function from mptt
        descendants = []
        for questionset in self.questionsets.all():
            descendants += questionset.get_descendants(include_self=True)

        if include_self:
            descendants.append(self)

        return descendants

    def get_ancestors(self, ascending=False, include_self=False):
        # this function tries to mimic the same function from mptt
        ancestors = []

        if include_self:
            ancestors.append(self)

        if self.questionset:
            ancestors += self.questionset.get_ancestors(ascending=True, include_self=True)

        if not ascending:
            ancestors.reverse()

        return ancestors

    @classmethod
    def build_path(cls, key, section, questionset=None):
        assert key
        assert section
        if questionset:
            return questionset.path + '/' + key
        else:
            return section.path + '/' + key

    @classmethod
    def build_uri(cls, uri_prefix, path):
        assert path
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/questions/', path)
